@@ -23,6 +23,7 @@
 
 #include "mutt.h"
 #include "mutt_menu.h"
+#include "attach.h"
 #include "mutt_curses.h"
 #include "keymap.h"
 #include "rfc1524.h"
@@ -255,19 +256,23 @@ int mutt_edit_attachment (BODY *a)
       {
 	/* For now, editing requires a file, no piping */
 	mutt_error _("Mailcap Edit entry requires %%s");
+        goto bailout;
       }
       else
       {
 	mutt_endwin (NULL);
 	if (mutt_system (command) == -1)
+        {
 	  mutt_error (_("Error running \"%s\"!"), command);
+          goto bailout;
+        }
       }
     }
   }
   else if (a->type == TYPETEXT)
   {
     /* On text, default to editor */
-    mutt_edit_file (NONULL (Editor), newfile);
+    mutt_edit_file (NONULL (Editor), a->filename);
   }
   else
   {
