@@ -166,6 +166,7 @@ int mutt_display_message (HEADER *cur)
   if (WithCrypto)
   {
     /* update crypto information for this message */
+    cur->security &= ~(GOODSIGN|BADSIGN);
     cur->security |= crypt_query (cur->content);
   
     /* Remove color cache for this message, in case there
@@ -200,7 +201,7 @@ int mutt_display_message (HEADER *cur)
 	mutt_message (_("PGP signature successfully verified."));
       else if (cur->security & PARTSIGN)
 	mutt_message (_("Warning: Part of this message has not been signed."));
-      else
+      else if (cur->security & SIGN)
 	mutt_message (_("PGP signature could NOT be verified."));
     }
 
@@ -670,21 +671,27 @@ static void set_copy_flags (HEADER *hdr, int decode, int decrypt, int *cmflags,
   }
 }
 
-void _mutt_save_message (HEADER *h, CONTEXT *ctx, int delete, int decode, int decrypt)
+int _mutt_save_message (HEADER *h, CONTEXT *ctx, int delete, int decode, int decrypt)
 {
   int cmflags, chflags;
+  int rc;
   
   set_copy_flags (h, decode, decrypt, &cmflags, &chflags);
 
   if (decode || decrypt)
     mutt_parse_mime_message (Context, h);
 
-  if (mutt_append_message (ctx, Context, h, cmflags, chflags) == 0 && delete)
+  if ((rc = mutt_append_message (ctx, Context, h, cmflags, chflags)) != 0)
+    return rc;
+
+  if (delete)
   {
     mutt_set_flag (Context, h, M_DELETE, 1);
     if (option (OPTDELETEUNTAG))
       mutt_set_flag (Context, h, M_TAG, 0);
   }
+  
+  return 0;
 }
 
 /* returns 0 if the copy/save was successful, or -1 on error/abort */
@@ -805,7 +812,13 @@ int mutt_save_message (HEADER *h, int delete,
   if (mx_open_mailbox (buf, M_APPEND, &ctx) != NULL)
   {
     if (h)
-      _mutt_save_message(h, &ctx, delete, decode, decrypt);
+    {
+      if (_mutt_save_message(h, &ctx, delete, decode, decrypt) != 0)
+      {
+        mx_close_mailbox (&ctx, NULL);
+        return -1;
+      }
+    }
     else
     {
       for (i = 0; i < Context->vcount; i++)
@@ -813,8 +826,12 @@ int mutt_save_message (HEADER *h, int delete,
 	if (Context->hdrs[Context->v2r[i]]->tagged)
 	{
 	  mutt_message_hook (Context, Context->hdrs[Context->v2r[i]], M_MESSAGEHOOK);
-	  _mutt_save_message(Context->hdrs[Context->v2r[i]],
-			     &ctx, delete, decode, decrypt);
+	  if (_mutt_save_message(Context->hdrs[Context->v2r[i]],
+			     &ctx, delete, decode, decrypt) != 0)
+          {
+            mx_close_mailbox (&ctx, NULL);
+            return -1;
+          }
 	}
       }
     }
