@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -24,12 +24,16 @@
 #define M_CM_UPDATE	(1<<4)  /* update structs on sync */
 #define M_CM_WEED	(1<<5)  /* weed message/rfc822 attachment headers */
 #define M_CM_CHARCONV	(1<<6)  /* perform character set conversions */
+#define M_CM_PRINTING	(1<<7)	/* printing the message - display light */
+#define M_CM_REPLYING	(1<<8)	/* replying the message */
 
 
-#ifdef HAVE_PGP
-#define M_CM_DECODE_PGP	(1<<7)	/* used for decoding PGP messages */
-#define M_CM_VERIFY	(1<<8)	/* do signature verification */
-#endif
+#define M_CM_DECODE_PGP	  (1<<9) /* used for decoding PGP messages */
+#define M_CM_DECODE_SMIME (1<<10) /* used for decoding S/MIME messages */
+#define M_CM_DECODE_CRYPT  (M_CM_DECODE_PGP | M_CM_DECODE_SMIME)
+
+
+#define M_CM_VERIFY	(1<<11)	/* do signature verification */
 
 
 
