@@ -138,16 +138,14 @@ int _mutt_get_field (/* const */ char *field, char *buf, size_t buflen, int comp
   return (ret);
 }
 
-int mutt_get_password (char *msg, char *buf, size_t buflen)
+int mutt_get_field_unbuffered (char *msg, char *buf, size_t buflen, int flags)
 {
   int rc;
-  
-  CLEARLINE (LINES-1);
-  addstr (msg);
+
   set_option (OPTUNBUFFEREDINPUT);
-  rc = mutt_enter_string (buf, buflen, LINES - 1, mutt_strlen (msg), M_PASS);
+  rc = mutt_get_field (msg, buf, buflen, flags);
   unset_option (OPTUNBUFFEREDINPUT);
-  CLEARLINE (LINES-1);
+
   return (rc);
 }
 
@@ -342,6 +340,38 @@ void mutt_curses_message (const char *fmt, ...)
   unset_option (OPTMSGERR);
 }
 
+#ifdef USE_SOCKET
+void mutt_progress_bar (progress_t* progress, long pos)
+{
+  char posstr[SHORT_STRING];
+
+  if (!pos)
+  {
+    if (!NetInc)
+      mutt_message (progress->msg);
+    else {
+      if (progress->size)
+	mutt_pretty_size (progress->sizestr, sizeof (progress->sizestr), progress->size);
+      progress->pos = 0;
+    }
+  }
+
+  if (!NetInc)
+    return;
+
+  if (pos >= progress->pos + (NetInc << 10))
+  {
+    progress->pos = pos;
+    pos = pos / (NetInc << 10) * (NetInc << 10);
+    mutt_pretty_size (posstr, sizeof (posstr), pos);
+    if (progress->size)
+      mutt_message ("%s %s/%s", progress->msg, posstr, progress->sizestr);
+    else
+      mutt_message ("%s %s", progress->msg, posstr);
+  }
+}
+#endif
+
 void mutt_show_error (void)
 {
   if (option (OPTKEEPQUIET))
