@@ -23,6 +23,7 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
+#include "attach.h"
 #include "mailbox.h"
 #include "mapping.h"
 #include "sort.h"
@@ -54,6 +55,13 @@ static const char *Mailbox_is_read_only = N_("Mailbox is read-only.");
 static const char *Function_not_permitted_in_attach_message_mode = N_("Function not permitted in attach-message mode.");
 static const char *No_visible = N_("No visible messages.");
 
+#define CHECK_IN_MAILBOX if (!Context) \
+	{ \
+		mutt_flushinp (); \
+		mutt_error _(No_mailbox_is_open); \
+		break; \
+	}
+
 #define CHECK_MSGCOUNT if (!Context) \
 	{ \
 	  	mutt_flushinp (); \
@@ -285,20 +293,17 @@ static void update_index (MUTTMENU *menu, CONTEXT *ctx, int check,
   if (Context->pattern)
   {
 #define THIS_BODY Context->hdrs[j]->content
-    if (oldcount || check == M_REOPENED)
+    for (j = (check == M_REOPENED) ? 0 : oldcount; j < Context->msgcount; j++)
     {
-      for (j = (check == M_REOPENED) ? 0 : oldcount; j < Context->msgcount; j++)
+      if (mutt_pattern_exec (Context->limit_pattern,
+			     M_MATCH_FULL_ADDRESS, 
+			     Context, Context->hdrs[j]))
       {
-	if (mutt_pattern_exec (Context->limit_pattern,
-			       M_MATCH_FULL_ADDRESS, 
-			       Context, Context->hdrs[j]))
-	{
-	  Context->hdrs[j]->virtual = Context->vcount;
-	  Context->v2r[Context->vcount] = j;
-	  Context->hdrs[j]->limited = 1;
-	  Context->vcount++;
-	  Context->vsize += THIS_BODY->length + THIS_BODY->offset - THIS_BODY->hdr_offset;
-	}
+	Context->hdrs[j]->virtual = Context->vcount;
+	Context->v2r[Context->vcount] = j;
+	Context->hdrs[j]->limited = 1;
+	Context->vcount++;
+	Context->vsize += THIS_BODY->length + THIS_BODY->offset - THIS_BODY->hdr_offset;
       }
     }
 #undef THIS_BODY
@@ -563,8 +568,10 @@ int mutt_index_menu (void)
 	menu_status_line (buf, sizeof (buf), menu, NONULL (Status));
 	CLEARLINE (option (OPTSTATUSONTOP) ? 0 : LINES-2);
 	SETCOLOR (MT_COLOR_STATUS);
+        BKGDSET (MT_COLOR_STATUS);
 	mutt_paddstr (COLS, buf);
 	SETCOLOR (MT_COLOR_NORMAL);
+        BKGDSET (MT_COLOR_NORMAL);
 	menu->redraw &= ~REDRAW_STATUS;
       }
 
@@ -576,6 +583,8 @@ int mutt_index_menu (void)
 
       if (option (OPTARROWCURSOR))
 	move (menu->current - menu->top + menu->offset, 2);
+      else if (option (OPTBRAILLEFRIENDLY))
+	move (menu->current - menu->top + menu->offset, 0);
       else
 	move (menu->current - menu->top + menu->offset, COLS - 1);
       mutt_refresh ();
@@ -821,7 +830,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	break;
 
       case OP_MAIN_SHOW_LIMIT:
-        CHECK_MSGCOUNT;
+	CHECK_IN_MAILBOX;
 	if (!Context->pattern)
 	   mutt_message _("No limit pattern is in effect.");
 	else
@@ -835,7 +844,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 
       case OP_MAIN_LIMIT:
 
-	CHECK_MSGCOUNT;
+	CHECK_IN_MAILBOX;
 	menu->oldcurrent = (Context->vcount && menu->current >= 0 && menu->current < Context->vcount) ?
 		CURHDR->index : -1;
 	if (mutt_pattern_func (M_LIMIT, _("Limit to messages matching: ")) == 0)
@@ -855,7 +864,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	  else
 	    menu->current = 0;
 	  menu->redraw = REDRAW_INDEX | REDRAW_STATUS;
-	  if ((Sort & SORT_MASK) == SORT_THREADS)
+	  if (Context->msgcount && (Sort & SORT_MASK) == SORT_THREADS)
 	    mutt_draw_tree (Context);
 	  menu->redraw = REDRAW_FULL;
 	}
@@ -934,6 +943,11 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	else
 	{
 	  mutt_set_flag (Context, CURHDR, M_TAG, !CURHDR->tagged);
+
+	  Context->last_tag = CURHDR->tagged ? CURHDR :
+	    ((Context->last_tag == CURHDR && !CURHDR->tagged)
+	     ? NULL : Context->last_tag);
+
 	  menu->redraw = REDRAW_STATUS;
 	  if (option (OPTRESOLVE) && menu->current < Context->vcount - 1)
 	  {
@@ -1071,6 +1085,7 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	  mutt_error (_("%s is not a mailbox."), buf);
 	  break;
 	}
+	mutt_str_replace (&CurrentFolder, buf);
 
         if (Context)
         {
@@ -1173,6 +1188,77 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
 	}
 	break;
 
+      case OP_MAIN_BREAK_THREAD:
+
+	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	CHECK_READONLY;
+
+        if ((Sort & SORT_MASK) != SORT_THREADS)
+	  mutt_error _("Threading is not enabled.");
+	else
+	{
+	  {
+	    HEADER *oldcur = CURHDR;
+
+	    mutt_break_thread (CURHDR);
+	    mutt_sort_headers (Context, 1);
+	    menu->current = oldcur->virtual;
+	  }
+
+	  Context->changed = 1;
+	  mutt_message _("Thread broken");
+
+	  if (menu->menu == MENU_PAGER)
+	  {
+	    op = OP_DISPLAY_MESSAGE;
+	    continue;
+	  }
+	  else
+	    menu->redraw |= REDRAW_INDEX;
+	}
+
+	  break;
+
+      case OP_MAIN_LINK_THREADS:
+
+	CHECK_MSGCOUNT;
+        CHECK_VISIBLE;
+	CHECK_READONLY;
+
+        if ((Sort & SORT_MASK) != SORT_THREADS)
+	  mutt_error _("Threading is not enabled.");
+	else if (!CURHDR->env->message_id)
+	  mutt_error _("No Message-ID: header available to link thread");
+	else if (!tag && (!Context->last_tag || !Context->last_tag->tagged))
+	  mutt_error _("First, please tag a message to be linked here");
+	else 
+	{
+	  HEADER *oldcur = CURHDR;
+
+	  if (mutt_link_threads (CURHDR, tag ? NULL : Context->last_tag,
+				 Context))
+	  {
+	    mutt_sort_headers (Context, 1);
+	    menu->current = oldcur->virtual;
+	    
+	    Context->changed = 1;
+	    mutt_message _("Threads linked");
+	  }
+	  else
+	    mutt_error _("No thread linked");
+	}
+
+	if (menu->menu == MENU_PAGER)
+	{
+	  op = OP_DISPLAY_MESSAGE;
+	  continue;
+	}
+	else
+	  menu->redraw |= REDRAW_STATUS | REDRAW_INDEX;
+
+	break;
+
       case OP_EDIT_TYPE:
 
 	CHECK_MSGCOUNT;
@@ -1500,7 +1586,7 @@ CHECK_IMAP_ACL(IMAP_ACL_SEEN);
 
       case OP_TOGGLE_WRITE:
 
-	CHECK_MSGCOUNT;
+	CHECK_IN_MAILBOX;
 	if (mx_toggle_write (Context) == 0)
 	  menu->redraw |= REDRAW_STATUS;
 	break;
@@ -2109,12 +2195,6 @@ CHECK_IMAP_ACL(IMAP_ACL_DELETE);
     if (done) break;
   }
 
-#ifdef USE_IMAP
-  /* Close all open IMAP connections */
-  if (!attach_msg)
-    imap_logout_all ();
-#endif
-
   mutt_menuDestroy (&menu);
   return (close);
 }
