@@ -28,6 +28,7 @@ WHERE char AttachmentMarker[STRING];
 WHERE char *MuttDotlock;
 #endif
 
+WHERE ADDRESS *EnvFrom;
 WHERE ADDRESS *From;
 
 WHERE char *AliasFile;
@@ -57,6 +58,7 @@ WHERE char *ImapAuthenticators INITVAL (NULL);
 WHERE char *ImapDelimChars INITVAL (NULL);
 WHERE char *ImapHeaders;
 WHERE char *ImapHomeNamespace INITVAL (NULL);
+WHERE char *ImapLogin INITVAL (NULL);
 WHERE char *ImapPass INITVAL (NULL);
 WHERE char *ImapUser INITVAL (NULL);
 #endif
@@ -77,6 +79,7 @@ WHERE char *MsgFmt;
 #ifdef USE_SOCKET
 WHERE char *Preconnect INITVAL (NULL);
 WHERE char *Tunnel INITVAL (NULL);
+WHERE short NetInc;
 #endif /* USE_SOCKET */
 
 #ifdef MIXMASTER
@@ -109,13 +112,14 @@ WHERE char *Signature;
 WHERE char *SimpleSearch;
 WHERE char *Spoolfile;
 WHERE char *SpamSep;
-#if defined(USE_SSL) || defined(USE_NSS) || defined(USE_GNUTLS)
+#if defined(USE_SSL) || defined(USE_GNUTLS)
 WHERE char *SslCertFile INITVAL (NULL);
 #endif
 #ifdef USE_SSL
 WHERE char *SslClientCert INITVAL (NULL);
+WHERE LIST *SslSessionCerts INITVAL (NULL);
 #endif
-#if defined(USE_SSL) || defined(USE_NSS)
+#if defined(USE_SSL)
 WHERE char *SslEntropyFile INITVAL (NULL);
 #endif
 #ifdef USE_GNUTLS
@@ -129,6 +133,7 @@ WHERE char *Tochars;
 WHERE char *Username;
 WHERE char *Visual;
 
+WHERE char *CurrentFolder;
 WHERE char *LastFolder;
 
 WHERE LIST *AutoViewList INITVAL(0);
