@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -16,6 +16,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -30,20 +34,14 @@
 #include "mime.h"
 #include "copy.h"
 #include "charset.h"
-
-
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
+#include "mutt_crypt.h"
 
 
 #define BUFI_SIZE 1000
 #define BUFO_SIZE 2000
 
 
-typedef void handler_f (BODY *, STATE *);
-typedef handler_f *handler_t;
+typedef int (*handler_t) (BODY *, STATE *);
 
 int Index_hex[128] = {
     -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
@@ -76,10 +74,10 @@ static void state_prefix_put (const char *d, size_t dlen, STATE *s)
     fwrite (d, dlen, 1, s->fpout);
 }
 
-static void convert_to_state(iconv_t cd, char *bufi, size_t *l, STATE *s)
+void mutt_convert_to_state(iconv_t cd, char *bufi, size_t *l, STATE *s)
 {
   char bufo[BUFO_SIZE];
-  const char *ib;
+  ICONV_CONST char *ib;
   char *ob;
   size_t ibl, obl;
 
@@ -140,11 +138,11 @@ void mutt_decode_xbit (STATE *s, long len, int istext, iconv_t cd)
 
       bufi[l++] = c;
       if (l == sizeof (bufi))
-	convert_to_state (cd, bufi, &l, s);
+	mutt_convert_to_state (cd, bufi, &l, s);
     }
 
-    convert_to_state (cd, bufi, &l, s);
-    convert_to_state (cd, 0, 0, s);
+    mutt_convert_to_state (cd, bufi, &l, s);
+    mutt_convert_to_state (cd, 0, 0, s);
 
     state_reset_prefix (s);
   }
@@ -159,7 +157,9 @@ static int qp_decode_triple (char *s, char *d)
     return 1;
   
   /* quoted-printable triple */
-  if (*s == '=' && isxdigit (*(s+1)) && isxdigit (*(s+2)))
+  if (*s == '=' &&
+      isxdigit ((unsigned char) *(s+1)) &&
+      isxdigit ((unsigned char) *(s+2)))
   {
     *d = (hexval (*(s+1)) << 4) | hexval (*(s+2));
     return 0;
@@ -178,16 +178,6 @@ static void qp_decode_line (char *dest, char *src, size_t *l,
   int kind;
   int soft = 0;
 
-  /* chop trailing whitespace */
-
-  if (*src)
-  {
-    s = src + strlen(src) - 1;
-    while (s >= src && ISSPACE(*s))
-      s--;
-    *(++s) = '\0';
-  }
-
   /* decode the line */
   
   for (d = dest, s = src; *s;)
@@ -222,7 +212,7 @@ static void qp_decode_line (char *dest, char *src, size_t *l,
  * result of qp_decode_line.
  * 
  * Finally, at soft line breaks, some part of a multibyte character
- * may have been left over by convert_to_state().  This shouldn't
+ * may have been left over by mutt_convert_to_state().  This shouldn't
  * be more than 6 characters, so STRING + 7 should be sufficient
  * memory to store the decoded data.
  * 
@@ -236,10 +226,10 @@ void mutt_decode_quoted (STATE *s, long len, int istext, iconv_t cd)
   char line[STRING];
   char decline[2*STRING];
   size_t l = 0;
-  size_t l2;
+  size_t linelen;      /* number of input bytes in `line' */
   size_t l3;
   
-  int last;
+  int last;    /* store the last character in the input line */
   
   if (istext)
     state_set_prefix(s);
@@ -248,31 +238,40 @@ void mutt_decode_quoted (STATE *s, long len, int istext, iconv_t cd)
   {
     last = 0;
     
-    if (fgets (line, MIN (sizeof (line), len + 1), s->fpin) == NULL)
+    /*
+     * It's ok to use a fixed size buffer for input, even if the line turns
+     * out to be longer than this.  Just process the line in chunks.  This
+     * really shouldn't happen according the MIME spec, since Q-P encoded
+     * lines are at most 76 characters, but we should be liberal about what
+     * we accept.
+     */
+    if (fgets (line, MIN ((ssize_t)sizeof (line), len + 1), s->fpin) == NULL)
       break;
 
-    len -= (l2 = strlen (line));
+    linelen = strlen(line);
+    len -= linelen;
 
-    /* 
-     * Skip over pending input data until end of line - at this
-     * point, the only thing we may see is trailing whitespace,
-     * i.e. garbage.
+    /*
+     * inspect the last character we read so we can tell if we got the
+     * entire line.
      */
+    last = linelen ? line[linelen - 1] : 0;
 
-    if (l2 && (last = line[l2 - 1]) != '\n')
-      while (len > 0 && (last = fgetc (s->fpin)) != '\n')
-	len--;
+    /* chop trailing whitespace if we got the full line */
+    if (last == '\n')
+    {
+      while (linelen > 0 && ISSPACE (line[linelen-1]))
+       linelen--;
+      line[linelen]=0;
+    }
 
-    /* 
-     * decode and do character set conversion
-     */
-    
+    /* decode and do character set conversion */
     qp_decode_line (decline + l, line, &l3, last);
     l += l3;
-    convert_to_state (cd, decline, &l, s);
+    mutt_convert_to_state (cd, decline, &l, s);
   }
 
-  convert_to_state (cd, 0, 0, s);
+  mutt_convert_to_state (cd, 0, 0, s);
   state_reset_prefix(s);
 }
 
@@ -293,8 +292,8 @@ void mutt_decode_base64 (STATE *s, long len, int istext, iconv_t cd)
     for (i = 0 ; i < 4 && len > 0 ; len--)
     {
       if ((ch = fgetc (s->fpin)) == EOF)
-	return;
-      if (!ISSPACE (ch))
+	break;
+      if (ch >= 0 && ch < 128 && (base64val(ch) != -1 || ch == '='))
 	buf[i++] = ch;
     }
     if (i != 4)
@@ -347,13 +346,13 @@ void mutt_decode_base64 (STATE *s, long len, int istext, iconv_t cd)
       bufi[l++] = ch;
     
     if (l + 8 >= sizeof (bufi))
-      convert_to_state (cd, bufi, &l, s);
+      mutt_convert_to_state (cd, bufi, &l, s);
   }
 
   if (cr) bufi[l++] = '\r';
 
-  convert_to_state (cd, bufi, &l, s);
-  convert_to_state (cd, 0, 0, s);
+  mutt_convert_to_state (cd, bufi, &l, s);
+  mutt_convert_to_state (cd, 0, 0, s);
 
   state_reset_prefix(s);
 }
@@ -406,13 +405,13 @@ void mutt_decode_uuencoded (STATE *s, long len, int istext, iconv_t cd)
 	if (c == linelen)
 	  break;
       }
-      convert_to_state (cd, bufi, &k, s);
+      mutt_convert_to_state (cd, bufi, &k, s);
       pt++;
     }
   }
 
-  convert_to_state (cd, bufi, &k, s);
-  convert_to_state (cd, 0, 0, s);
+  mutt_convert_to_state (cd, bufi, &k, s);
+  mutt_convert_to_state (cd, 0, 0, s);
   
   state_reset_prefix(s);
 }
@@ -591,7 +590,7 @@ static void enriched_flush (struct enriched_state *stte, int wrap)
     if (stte->line_used > stte->line_max)
     {
       stte->line_max = stte->line_used;
-      safe_realloc ((void **) &stte->line, stte->line_max + 1);
+      safe_realloc (&stte->line, stte->line_max + 1);
     }
     strcat (stte->line, stte->buffer);	/* __STRCAT_CHECKED__ */
     stte->line_len += stte->word_len;
@@ -610,7 +609,7 @@ static void enriched_putc (int c, struct enriched_state *stte)
     if (stte->tag_level[RICH_COLOR]) 
     {
       if (stte->param_used + 1 >= stte->param_len)
-	safe_realloc ((void **) &stte->param, (stte->param_len += STRING));
+	safe_realloc (&stte->param, (stte->param_len += STRING));
 
       stte->param[stte->param_used++] = c;
     }
@@ -621,7 +620,7 @@ static void enriched_putc (int c, struct enriched_state *stte)
   if (stte->buff_len < stte->buff_used + 3)
   {
     stte->buff_len += LONG_STRING;
-    safe_realloc ((void **) &stte->buffer, stte->buff_len + 1);
+    safe_realloc (&stte->buffer, stte->buff_len + 1);
   }
 
   if ((!stte->tag_level[RICH_NOFILL] && ISSPACE (c)) || c == '\0' )
@@ -677,7 +676,7 @@ static void enriched_puts (char *s, struct enriched_state *stte)
   if (stte->buff_len < stte->buff_used + mutt_strlen(s))
   {
     stte->buff_len += LONG_STRING;
-    safe_realloc ((void **) &stte->buffer, stte->buff_len + 1);
+    safe_realloc (&stte->buffer, stte->buff_len + 1);
   }
   c = s;
   while (*c)
@@ -767,7 +766,7 @@ static void enriched_set_flags (const char *tag, struct enriched_state *stte)
   }
 }
 
-void text_enriched_handler (BODY *a, STATE *s)
+int text_enriched_handler (BODY *a, STATE *s)
 {
   enum {
     TEXT, LANGLE, TAG, BOGUS_TAG, NEWLINE, ST_EOF, DONE
@@ -889,6 +888,8 @@ void text_enriched_handler (BODY *a, STATE *s)
   FREE (&(stte.buffer));
   FREE (&(stte.line));
   FREE (&(stte.param));
+
+  return 0;
 }                                                                              
 
 /*
@@ -936,7 +937,8 @@ static void flowed_stuff (STATE *s, char *cont, int level)
     if (*cont && !level && !mutt_strcmp (Pager, "builtin") && flowed_maybe_quoted (cont))
       state_puts ("\033[0m",s);
   }
-  else if ((*cont == ' ') || (*cont == '>') || (!level && !mutt_strncmp (cont, "From ", 5)))
+  else if ((!(s->flags & M_PRINTING)) && 
+	   ((*cont == ' ') || (*cont == '>') || (!level && !mutt_strncmp (cont, "From ", 5))))
     state_putc (' ', s);
 }
 
@@ -962,7 +964,7 @@ static int flowed_visual_strlen (char *l, int i)
   return j;
 }
 
-static void text_plain_flowed_handler (BODY *a, STATE *s)
+static int text_plain_flowed_handler (BODY *a, STATE *s)
 {
   char line[LONG_STRING];
   char indent[LONG_STRING];
@@ -1184,7 +1186,8 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
 
   if (col)
     state_putc ('\n', s);
-  
+
+  return 0;
 }
 
 
@@ -1196,13 +1199,32 @@ static void text_plain_flowed_handler (BODY *a, STATE *s)
 #define TXTPLAIN    2
 #define TXTENRICHED 3
 
-static void alternative_handler (BODY *a, STATE *s)
+static int alternative_handler (BODY *a, STATE *s)
 {
   BODY *choice = NULL;
   BODY *b;
   LIST *t;
   char buf[STRING];
   int type = 0;
+  int mustfree = 0;
+  int rc = 0;
+
+  if (a->encoding == ENCBASE64 || a->encoding == ENCQUOTEDPRINTABLE ||
+      a->encoding == ENCUUENCODED)
+  {
+    struct stat st;
+    mustfree = 1;
+    fstat (fileno (s->fpin), &st);
+    b = mutt_new_body ();
+    b->length = (long) st.st_size;
+    b->parts = mutt_parse_multipart (s->fpin,
+		  mutt_get_parameter ("boundary", a->parameter),
+		  (long) st.st_size, ascii_strcasecmp ("digest", a->subtype) == 0);
+  }
+  else
+    b = a;
+
+  a = b;
 
   /* First, search list of prefered types */
   t = AlternativeOrderList;
@@ -1328,15 +1350,22 @@ static void alternative_handler (BODY *a, STATE *s)
     /* didn't find anything that we could display! */
     state_mark_attach (s);
     state_puts(_("[-- Error:  Could not display any parts of Multipart/Alternative! --]\n"), s);
+    rc = -1;
   }
+
+  if (mustfree)
+    mutt_free_body(&a);
+
+  return rc;
 }
 
 /* handles message/rfc822 body parts */
-void message_handler (BODY *a, STATE *s)
+int message_handler (BODY *a, STATE *s)
 {
   struct stat st;
   BODY *b;
   long off_start;
+  int rc = 0;
 
   off_start = ftell (s->fpin);
   if (a->encoding == ENCBASE64 || a->encoding == ENCQUOTEDPRINTABLE || 
@@ -1360,12 +1389,14 @@ void message_handler (BODY *a, STATE *s)
       state_puts (s->prefix, s);
     state_putc ('\n', s);
 
-    mutt_body_handler (b->parts, s);
+    rc = mutt_body_handler (b->parts, s);
   }
 
   if (a->encoding == ENCBASE64 || a->encoding == ENCQUOTEDPRINTABLE ||
       a->encoding == ENCUUENCODED)
     mutt_free_body (&b);
+  
+  return rc;
 }
 
 /* returns 1 if decoding the attachment will produce output */
@@ -1382,50 +1413,40 @@ int mutt_can_decode (BODY *a)
     return (1);
   else if (a->type == TYPEMULTIPART)
   {
+    BODY *p;
 
-
-
-#ifdef HAVE_PGP
-    if (ascii_strcasecmp (a->subtype, "signed") == 0 ||
-	ascii_strcasecmp (a->subtype, "encrypted") == 0)
-      return (1);
-    else
-#endif
-
-
-
+    if (WithCrypto)
     {
-      BODY *p;
-
-      for (p = a->parts; p; p = p->next)
-      {
-	if (mutt_can_decode (p))
-	  return (1);
-      }
+      if (ascii_strcasecmp (a->subtype, "signed") == 0 ||
+	  ascii_strcasecmp (a->subtype, "encrypted") == 0)
+        return (1);
     }
+
+    for (p = a->parts; p; p = p->next)
+    {
+      if (mutt_can_decode (p))
+        return (1);
+    }
+    
   }
-
-
-
-#ifdef HAVE_PGP
-  else if (a->type == TYPEAPPLICATION)
+  else if (WithCrypto && a->type == TYPEAPPLICATION)
   {
-    if (mutt_is_application_pgp(a))
+    if ((WithCrypto & APPLICATION_PGP) && mutt_is_application_pgp(a))
+      return (1);
+    if ((WithCrypto & APPLICATION_SMIME) && mutt_is_application_smime(a))
       return (1);
   }
-#endif
-
-
 
   return (0);
 }
 
-void multipart_handler (BODY *a, STATE *s)
+int multipart_handler (BODY *a, STATE *s)
 {
   BODY *b, *p;
   char length[5];
   struct stat st;
   int count;
+  int rc = 0;
 
   if (a->encoding == ENCBASE64 || a->encoding == ENCQUOTEDPRINTABLE ||
       a->encoding == ENCUUENCODED)
@@ -1476,16 +1497,21 @@ void multipart_handler (BODY *a, STATE *s)
 	state_printf(s, "%s: \n", p->form_name);
 
     }
-    mutt_body_handler (p, s);
+    rc = mutt_body_handler (p, s);
     state_putc ('\n', s);
+    if (rc || ((s->flags & M_REPLYING)
+               && (option (OPTINCLUDEONLYFIRST)) && (s->flags & M_FIRSTDONE)))
+      break;
   }
 
   if (a->encoding == ENCBASE64 || a->encoding == ENCQUOTEDPRINTABLE ||
       a->encoding == ENCUUENCODED)
     mutt_free_body (&b);
+  
+  return rc;
 }
 
-void autoview_handler (BODY *a, STATE *s)
+int autoview_handler (BODY *a, STATE *s)
 {
   rfc1524_entry *entry = rfc1524_new_entry ();
   char buffer[LONG_STRING];
@@ -1498,6 +1524,7 @@ void autoview_handler (BODY *a, STATE *s)
   FILE *fperr = NULL;
   int piped = FALSE;
   pid_t thepid;
+  int rc = 0;
 
   snprintf (type, sizeof (type), "%s/%s", TYPE (a), a->subtype);
   rfc1524_mailcap_lookup (a, type, entry, M_AUTOVIEW);
@@ -1525,7 +1552,7 @@ void autoview_handler (BODY *a, STATE *s)
     {
       mutt_perror ("fopen");
       rfc1524_free_entry (&entry);
-      return;
+      return -1;
     }
     
     mutt_copy_bytes (s->fpin, fpin, a->length);
@@ -1552,6 +1579,7 @@ void autoview_handler (BODY *a, STATE *s)
 	state_mark_attach (s);
 	state_printf (s, _("[-- Can't run %s. --]\n"), command);
       }
+      rc = -1;
       goto bail;
     }
     
@@ -1612,9 +1640,11 @@ void autoview_handler (BODY *a, STATE *s)
       mutt_clear_error ();
   }
   rfc1524_free_entry (&entry);
+
+  return rc;
 }
 
-static void external_body_handler (BODY *b, STATE *s)
+static int external_body_handler (BODY *b, STATE *s)
 {
   const char *access_type;
   const char *expiration;
@@ -1628,7 +1658,7 @@ static void external_body_handler (BODY *b, STATE *s)
       state_mark_attach (s);
       state_puts (_("[-- Error: message/external-body has no access-type parameter --]\n"), s);
     }
-    return;
+    return -1;
   }
 
   expiration = mutt_get_parameter ("expiration", b->parameter);
@@ -1704,11 +1734,13 @@ static void external_body_handler (BODY *b, STATE *s)
 		     CH_DECODE , NULL);
     }
   }
+  
+  return 0;
 }
 
 void mutt_decode_attachment (BODY *b, STATE *s)
 {
-  int istext = mutt_is_text_type (b->type, b->subtype);
+  int istext = mutt_is_text_part (b);
   iconv_t cd = (iconv_t)(-1);
 
   if (istext && s->flags & M_CHARCONV)
@@ -1739,7 +1771,7 @@ void mutt_decode_attachment (BODY *b, STATE *s)
     iconv_close (cd);
 }
 
-void mutt_body_handler (BODY *b, STATE *s)
+int mutt_body_handler (BODY *b, STATE *s)
 {
   int decode = 0;
   int plaintext = 0;
@@ -1749,6 +1781,7 @@ void mutt_body_handler (BODY *b, STATE *s)
   long tmpoffset = 0;
   size_t tmplength = 0;
   char type[STRING];
+  int rc = 0;
 
   int oflags = s->flags;
   
@@ -1773,14 +1806,16 @@ void mutt_body_handler (BODY *b, STATE *s)
       /* avoid copying this part twice since removing the transfer-encoding is
        * the only operation needed.
        */
-      if (ascii_strcasecmp ("flowed", mutt_get_parameter ("format", b->parameter)) == 0)
+      if ((WithCrypto & APPLICATION_PGP) && mutt_is_application_pgp (b))
+	handler = crypt_pgp_application_pgp_handler;
+      else if (ascii_strcasecmp ("flowed", mutt_get_parameter ("format", b->parameter)) == 0)
 	handler = text_plain_flowed_handler;
       else
 	plaintext = 1;
     }
     else if (ascii_strcasecmp ("enriched", b->subtype) == 0)
       handler = text_enriched_handler;
-    else if (ascii_strcasecmp ("rfc822-headers", b->subtype) == 0)
+    else /* text body type without a handler */
       plaintext = 1;
   }
   else if (b->type == TYPEMESSAGE)
@@ -1794,61 +1829,40 @@ void mutt_body_handler (BODY *b, STATE *s)
   }
   else if (b->type == TYPEMULTIPART)
   {
-
-
-
-#ifdef HAVE_PGP
     char *p;
-#endif /* HAVE_PGP */
-
-
 
     if (ascii_strcasecmp ("alternative", b->subtype) == 0)
       handler = alternative_handler;
-
-
-
-#ifdef HAVE_PGP
-    else if (ascii_strcasecmp ("signed", b->subtype) == 0)
+    else if (WithCrypto && ascii_strcasecmp ("signed", b->subtype) == 0)
     {
       p = mutt_get_parameter ("protocol", b->parameter);
 
       if (!p)
         mutt_error _("Error: multipart/signed has no protocol.");
-      else if (ascii_strcasecmp ("application/pgp-signature", p) == 0 ||
-	       ascii_strcasecmp ("multipart/mixed", p) == 0)
-      {
-	if (s->flags & M_VERIFY)
-	  handler = pgp_signed_handler;
-      }
+      else if (s->flags & M_VERIFY)
+	handler = mutt_signed_handler;
     }
-    else if (ascii_strcasecmp ("encrypted", b->subtype) == 0)
+    else if ((WithCrypto & APPLICATION_PGP)
+             && mutt_strcasecmp ("encrypted", b->subtype) == 0)
     {
       p = mutt_get_parameter ("protocol", b->parameter);
 
       if (!p)
         mutt_error _("Error: multipart/encrypted has no protocol parameter!");
       else if (ascii_strcasecmp ("application/pgp-encrypted", p) == 0)
-        handler = pgp_encrypted_handler;
+        handler = crypt_pgp_encrypted_handler;
     }
-#endif /* HAVE_PGP */
-
-
 
     if (!handler)
       handler = multipart_handler;
   }
-
-
-
-#ifdef HAVE_PGP
-  else if (b->type == TYPEAPPLICATION)
+  else if (WithCrypto && b->type == TYPEAPPLICATION)
   {
-    if (mutt_is_application_pgp(b))
-      handler = pgp_application_pgp_handler;
+    if ((WithCrypto & APPLICATION_PGP) && mutt_is_application_pgp (b))
+      handler = crypt_pgp_application_pgp_handler;
+    if ((WithCrypto & APPLICATION_SMIME) && mutt_is_application_smime(b))
+      handler = crypt_smime_application_smime_handler;
   }
-#endif /* HAVE_PGP */
-
 
 
   if (plaintext || handler)
@@ -1858,7 +1872,7 @@ void mutt_body_handler (BODY *b, STATE *s)
     /* see if we need to decode this part before processing it */
     if (b->encoding == ENCBASE64 || b->encoding == ENCQUOTEDPRINTABLE ||
 	b->encoding == ENCUUENCODED || plaintext || 
-	mutt_is_text_type (b->type, b->subtype))	/* text subtypes may
+	mutt_is_text_part (b))				/* text subtypes may
 							 * require character
 							 * set conversion even
 							 * with 8bit encoding.
@@ -1918,7 +1932,7 @@ void mutt_body_handler (BODY *b, STATE *s)
     /* process the (decoded) body part */
     if (handler)
     {
-      handler (b, s);
+      rc = handler (b, s);
 
       if (decode)
       {
@@ -1930,6 +1944,7 @@ void mutt_body_handler (BODY *b, STATE *s)
 	s->fpin = fp;
       }
     }
+    s->flags |= M_FIRSTDONE;
   }
   else if (s->flags & M_DISPLAY)
   {
@@ -1945,7 +1960,9 @@ void mutt_body_handler (BODY *b, STATE *s)
     }
     fputs (" --]\n", s->fpout);
   }
-  
+
   bail:
-  s->flags = oflags;
+  s->flags = oflags | (s->flags & M_FIRSTDONE);
+
+  return rc;
 }
