@@ -30,6 +30,9 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#if HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
 #include "mutt.h"
 #ifdef USE_IMAP
 #include "message.h"
@@ -60,7 +63,7 @@ static struct header_cache
 typedef union
 {
   struct timeval timeval;
-  uint64_t uid_validity;
+  unsigned long uid_validity;
 } validate;
 
 static void *
@@ -447,7 +450,7 @@ generate_crc32()
   SPAM_LIST *sp = SpamList;
   RX_LIST *rx = NoSpamList;
 
-  crc = crc32(crc, (unsigned char const *) "$Id: hcache.c,v 3.4 2005/02/03 19:46:53 roessler Exp $", mutt_strlen("$Id: hcache.c,v 3.4 2005/02/03 19:46:53 roessler Exp $"));
+  crc = crc32(crc, (unsigned char const *) "$Id: hcache.c,v 3.7 2005/07/06 21:03:00 brendan Exp $", mutt_strlen("$Id: hcache.c,v 3.7 2005/07/06 21:03:00 brendan Exp $"));
 
 #if HAVE_LANGINFO_CODESET
   crc = crc32(crc, (unsigned char const *) Charset, mutt_strlen(Charset));
@@ -547,7 +550,7 @@ mutt_hcache_per_folder(const char *path, const char *folder)
  * db_store */
 static void *
 mutt_hcache_dump(void *_db, HEADER * h, int *off,
-		 uint64_t uid_validity)
+		 unsigned long uid_validity)
 {
   struct header_cache *db = _db;
   unsigned char *d = NULL;
@@ -556,7 +559,7 @@ mutt_hcache_dump(void *_db, HEADER * h, int *off,
   d = lazy_malloc(sizeof (validate));
 
   if (uid_validity)
-    memcpy(d, &uid_validity, sizeof (uint64_t));
+    memcpy(d, &uid_validity, sizeof (unsigned long));
   else
   {
     struct timeval now;
@@ -693,7 +696,7 @@ mutt_hcache_fetch(void *db, const char *filename,
 
 int
 mutt_hcache_store(void *db, const char *filename, HEADER * header,
-		  uint64_t uid_validity,
+		  unsigned long uid_validity,
 		  size_t(*keylen) (const char *fn))
 {
   struct header_cache *h = db;
@@ -802,7 +805,7 @@ mutt_hcache_open(const char *path, const char *folder)
     return NULL;
   }
 
-  ret = h->env->open(h->env, NULL, DB_INIT_MPOOL | DB_CREATE | DB_PRIVATE,
+  ret = (*h->env->open)(h->env, NULL, DB_INIT_MPOOL | DB_CREATE | DB_PRIVATE,
 	0600);
   if (!ret)
   {
@@ -823,7 +826,7 @@ mutt_hcache_open(const char *path, const char *folder)
     h->db->set_pagesize(h->db, pagesize);
   }
 
-  ret = h->db->open(h->db, NULL, path, folder, DB_BTREE, createflags, 0600);
+  ret = (*h->db->open)(h->db, NULL, path, folder, DB_BTREE, createflags, 0600);
   if (ret)
   {
     h->db->close(h->db, 0);
@@ -841,7 +844,6 @@ void
 mutt_hcache_close(void *db)
 {
   struct header_cache *h = db;
-  int ret;
 
   if (!h)
     return;
@@ -883,7 +885,7 @@ mutt_hcache_fetch(void *db, const char *filename,
 
 int
 mutt_hcache_store(void *db, const char *filename, HEADER * header,
-		  uint64_t uid_validity,
+		  unsigned long uid_validity,
 		  size_t(*keylen) (const char *fn))
 {
   DBT key;
