@@ -27,13 +27,8 @@
 #include "imap_private.h"
 #include "auth.h"
 
-#ifdef USE_SASL2
 #include <sasl/sasl.h>
 #include <sasl/saslutil.h>
-#else
-#include <sasl.h>
-#include <saslutil.h>
-#endif
 
 /* imap_auth_sasl: Default authenticator if available. */
 imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
@@ -43,11 +38,7 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
   int rc, irc;
   char buf[HUGE_STRING];
   const char* mech;
-#ifdef USE_SASL2
   const char *pc = NULL;
-#else
-  char* pc = NULL;
-#endif
   unsigned int len, olen;
   unsigned char client_start;
 
@@ -76,25 +67,15 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
     if (mutt_bit_isset (idata->capabilities, AUTH_ANON) &&
 	(!idata->conn->account.user[0] ||
 	 !ascii_strncmp (idata->conn->account.user, "anonymous", 9)))
-#ifdef USE_SASL2
       rc = sasl_client_start (saslconn, "AUTH=ANONYMOUS", NULL, &pc, &olen, 
                               &mech);
-#else
-      rc = sasl_client_start (saslconn, "AUTH=ANONYMOUS", NULL, NULL, &pc, &olen,
-			      &mech);
-#endif
   }
   
   if (rc != SASL_OK && rc != SASL_CONTINUE)
     do
     {
-#ifdef USE_SASL2
       rc = sasl_client_start (saslconn, method, &interaction,
         &pc, &olen, &mech);
-#else
-      rc = sasl_client_start (saslconn, method, NULL, &interaction,
-        &pc, &olen, &mech);
-#endif
       if (rc == SASL_INTERACT)
 	mutt_sasl_interact (interaction);
     }
@@ -138,19 +119,17 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
 
     if (irc == IMAP_CMD_RESPOND)
     {
-#ifdef USE_SASL2
-      if (sasl_decode64 (idata->cmd.buf+2, strlen (idata->cmd.buf+2), buf, LONG_STRING-1,
-#else
-      if (sasl_decode64 (idata->cmd.buf+2, strlen (idata->cmd.buf+2), buf,
-#endif
-			 &len) != SASL_OK)
+      if (sasl_decode64 (idata->cmd.buf+2, strlen (idata->cmd.buf+2), buf, LONG_STRING-1, &len) != SASL_OK)
       {
 	dprint (1, (debugfile, "imap_auth_sasl: error base64-decoding server response.\n"));
 	goto bail;
       }
     }
 
-    if (!client_start)
+    /* client-start is only available with the SASL-IR extension, but
+     * SASL 2.1 seems to want to use it regardless, at least for DIGEST
+     * fast reauth. Override if the server sent an initial continuation */
+    if (!client_start || buf[0])
     {
       do
       {
@@ -171,12 +150,6 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
 	dprint (1, (debugfile, "imap_auth_sasl: error base64-encoding client response.\n"));
 	goto bail;
       }
-
-      /* sasl_client_st(art|ep) allocate pc with malloc, expect me to 
-       * free it */
-#ifndef USE_SASL2
-      FREE (&pc);
-#endif
     }
     
     if (irc == IMAP_CMD_RESPOND)
