@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2003 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2005 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 #endif
 
 #include "mutt.h"
-#include "mutt_curses.h"
 #include "mx.h"
 #include "mailbox.h"
 #include "globals.h"
@@ -36,6 +35,7 @@
 #if defined(USE_SSL) || defined(USE_GNUTLS)
 # include "mutt_ssl.h"
 #endif
+#include "buffy.h"
 
 #include <unistd.h>
 #include <ctype.h>
@@ -52,7 +52,9 @@ static int imap_check_capabilities (IMAP_DATA* idata);
 static void imap_set_flag (IMAP_DATA* idata, int aclbit, int flag,
 			   const char* str, char* flags, size_t flsize);
 
-/* imap_access: Check permissions on an IMAP mailbox. */
+/* imap_access: Check permissions on an IMAP mailbox.
+ * TODO: ACL checks. Right now we assume if it exists we can
+ *       mess with it. */
 int imap_access (const char* path, int flags)
 {
   IMAP_DATA* idata;
@@ -72,10 +74,17 @@ int imap_access (const char* path, int flags)
   }
 
   imap_fix_path (idata, mx.mbox, mailbox, sizeof (mailbox));
+
+  /* we may already be in the folder we're checking */
+  if (!ascii_strcmp(idata->mailbox, mx.mbox))
+  {
+    FREE (&mx.mbox);
+    return 0;
+  }
+
   FREE (&mx.mbox);
   imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
 
-  /* TODO: ACL checks. Right now we assume if it exists we can mess with it. */
   if (mutt_bit_isset (idata->capabilities, IMAP4REV1))
     snprintf (buf, sizeof (buf), "STATUS %s (UIDVALIDITY)", mbox);
   else if (mutt_bit_isset (idata->capabilities, STATUS))
@@ -179,7 +188,7 @@ void imap_logout_all (void)
 /* imap_read_literal: read bytes bytes from server into file. Not explicitly
  *   buffered, relies on FILE buffering. NOTE: strips \r from \r\n.
  *   Apparently even literals use \r\n-terminated strings ?! */
-int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes)
+int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes, progress_t* pbar)
 {
   long pos;
   char c;
@@ -211,6 +220,9 @@ int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes)
       r = 0;
 #endif
     fputc (c, fp);
+    
+    if (pbar && !(pos % 1024))
+      mutt_progress_bar (pbar, pos);
 #ifdef DEBUG
     if (debuglevel >= IMAP_LOG_LTRL)
       fputc (c, debugfile);
@@ -341,6 +353,7 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
   CONNECTION* conn;
   IMAP_DATA* idata;
   ACCOUNT* creds;
+  int new = 0;
 
   if (!(conn = mutt_conn_find (NULL, account)))
     return NULL;
@@ -382,6 +395,7 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
 
     conn->data = idata;
     idata->conn = conn;
+    new = 1;
   }
 
   if (idata->state == IMAP_DISCONNECTED)
@@ -400,9 +414,16 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
     
     FREE (&idata->capstr);
   }
-  if (idata->state == IMAP_AUTHENTICATED)
+  if (new && idata->state == IMAP_AUTHENTICATED)
+  {
     imap_get_delim (idata);
-  
+    if (option (OPTIMAPCHECKSUBSCRIBED))
+    {
+      mutt_message _("Checking mailbox subscriptions");
+      imap_exec (idata, "LSUB \"\" \"*\"", 0);
+    }
+  }
+
   return idata;
 }
 
@@ -416,9 +437,6 @@ int imap_open_connection (IMAP_DATA* idata)
   idata->state = IMAP_CONNECTED;
 
   if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE) {
-    mutt_error (_("Unexpected response received from server: %s"), idata->cmd.buf);
-    mutt_sleep (1);
-
     mutt_socket_close (idata->conn);
     idata->state = IMAP_DISCONNECTED;
     return -1;
@@ -431,11 +449,14 @@ int imap_open_connection (IMAP_DATA* idata)
       goto bail;
 #if defined(USE_SSL) || defined(USE_GNUTLS)
     /* Attempt STARTTLS if available and desired. */
-    if (mutt_bit_isset (idata->capabilities, STARTTLS) && !idata->conn->ssf)
+    if (!idata->conn->ssf && (option(OPTSSLFORCETLS) ||
+                              mutt_bit_isset (idata->capabilities, STARTTLS)))
     {
       int rc;
-
-      if ((rc = query_quadoption (OPT_SSLSTARTTLS,
+      
+      if (option(OPTSSLFORCETLS))
+        rc = M_YES;
+      else if ((rc = query_quadoption (OPT_SSLSTARTTLS,
         _("Secure connection with TLS?"))) == -1)
 	goto err_close_conn;
       if (rc == M_YES) {
@@ -462,6 +483,13 @@ int imap_open_connection (IMAP_DATA* idata)
 	}
       }
     }
+
+    if (option(OPTSSLFORCETLS) && ! idata->conn->ssf)
+    {
+      mutt_error _("Encrypted connection unavailable");
+      mutt_sleep (1);
+      goto err_close_conn;
+    }
 #endif    
   }
   else if (ascii_strncasecmp ("* PREAUTH", idata->cmd.buf, 9) == 0)
@@ -635,7 +663,7 @@ int imap_open_mailbox (CONTEXT* ctx)
       dprint(2, (debugfile, "Getting mailbox UIDVALIDITY\n"));
       pc += 3;
       pc = imap_next_word(pc);
-      sscanf(pc, "%u", &(idata->uid_validity));
+      sscanf(pc, "%lu", &(idata->uid_validity));
     }
 #endif
     else
@@ -744,9 +772,8 @@ int imap_open_mailbox_append (CONTEXT *ctx)
 {
   CONNECTION *conn;
   IMAP_DATA *idata;
-  char buf[LONG_STRING], mbox[LONG_STRING];
+  char buf[LONG_STRING];
   char mailbox[LONG_STRING];
-  int r;
   IMAP_MBOX mx;
 
   if (imap_parse_path (ctx->path, &mx))
@@ -756,53 +783,31 @@ int imap_open_mailbox_append (CONTEXT *ctx)
    * ctx is brand new and mostly empty */
 
   if (!(idata = imap_conn_find (&(mx.account), 0)))
-    goto fail;
+  {
+    FREE (&mx.mbox);
+    return -1;
+  }
+
   conn = idata->conn;
 
   ctx->magic = M_IMAP;
   ctx->data = idata;
 
-  /* check mailbox existance */
-
   imap_fix_path (idata, mx.mbox, mailbox, sizeof (mailbox));
-
-  imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
-				
-  if (mutt_bit_isset(idata->capabilities,IMAP4REV1))
-    snprintf (buf, sizeof (buf), "STATUS %s (UIDVALIDITY)", mbox);
-  else if (mutt_bit_isset(idata->capabilities,STATUS))
-    /* We have no idea what the other guy wants. UW imapd 8.3 wants this
-     * (but it does not work if another mailbox is selected) */
-    snprintf (buf, sizeof (buf), "STATUS %s (UID-VALIDITY)", mbox);
-  else
-  {
-    /* STATUS not supported */
-    mutt_message _("Unable to append to IMAP mailboxes at this server");
-
-    goto fail;
-  }
-
-  r = imap_exec (idata, buf, IMAP_CMD_FAIL_OK);
-  if (r == -2)
-  {
-    /* command failed cause folder doesn't exist */
-    snprintf (buf, sizeof (buf), _("Create %s?"), mailbox);
-    if (option (OPTCONFIRMCREATE) && mutt_yesorno (buf, 1) < 1)
-      goto fail;
-
-    if (imap_create_mailbox (idata, mailbox) < 0)
-      goto fail;
-  }
-  else if (r == -1)
-    /* Hmm, some other failure */
-    goto fail;
-
   FREE (&mx.mbox);
+
+  /* really we should also check for W_OK */
+  if (!imap_access (ctx->path, F_OK))
+    return 0;
+
+  snprintf (buf, sizeof (buf), _("Create %s?"), mailbox);
+  if (option (OPTCONFIRMCREATE) && mutt_yesorno (buf, 1) < 1)
+    return -1;
+
+  if (imap_create_mailbox (idata, mailbox) < 0)
+    return -1;
+
   return 0;
-
- fail:
-  FREE (&mx.mbox);
-  return -1;
 }
 
 /* imap_logout: Gracefully log out of server. */
@@ -921,6 +926,76 @@ int imap_make_msg_set (IMAP_DATA* idata, BUFFER* buf, int flag, int changed)
   return count;
 }
 
+/* Update the IMAP server to reflect the flags a single message.  */
+
+int imap_sync_message (IMAP_DATA *idata, HEADER *hdr, BUFFER *cmd,
+		       int *err_continue)
+{
+  char flags[LONG_STRING];
+  char uid[11];
+
+  hdr->changed = 0;
+
+  snprintf (uid, sizeof (uid), "%u", HEADER_DATA(hdr)->uid);
+  cmd->dptr = cmd->data;
+  mutt_buffer_addstr (cmd, "UID STORE ");
+  mutt_buffer_addstr (cmd, uid);
+
+  flags[0] = '\0';
+      
+  imap_set_flag (idata, IMAP_ACL_SEEN, hdr->read, "\\Seen ",
+		 flags, sizeof (flags));
+  imap_set_flag (idata, IMAP_ACL_WRITE, hdr->flagged,
+		 "\\Flagged ", flags, sizeof (flags));
+  imap_set_flag (idata, IMAP_ACL_WRITE, hdr->replied,
+		 "\\Answered ", flags, sizeof (flags));
+  imap_set_flag (idata, IMAP_ACL_DELETE, hdr->deleted,
+		 "\\Deleted ", flags, sizeof (flags));
+
+  /* now make sure we don't lose custom tags */
+  if (mutt_bit_isset (idata->rights, IMAP_ACL_WRITE))
+    imap_add_keywords (flags, hdr, idata->flags, sizeof (flags));
+
+  mutt_remove_trailing_ws (flags);
+
+  /* UW-IMAP is OK with null flags, Cyrus isn't. The only solution is to
+   * explicitly revoke all system flags (if we have permission) */
+  if (!*flags)
+  {
+    imap_set_flag (idata, IMAP_ACL_SEEN, 1, "\\Seen ", flags, sizeof (flags));
+    imap_set_flag (idata, IMAP_ACL_WRITE, 1, "\\Flagged ", flags, sizeof (flags));
+    imap_set_flag (idata, IMAP_ACL_WRITE, 1, "\\Answered ", flags, sizeof (flags));
+    imap_set_flag (idata, IMAP_ACL_DELETE, 1, "\\Deleted ", flags, sizeof (flags));
+
+    mutt_remove_trailing_ws (flags);
+
+    mutt_buffer_addstr (cmd, " -FLAGS.SILENT (");
+  } else
+    mutt_buffer_addstr (cmd, " FLAGS.SILENT (");
+
+  mutt_buffer_addstr (cmd, flags);
+  mutt_buffer_addstr (cmd, ")");
+
+  /* dumb hack for bad UW-IMAP 4.7 servers spurious FLAGS updates */
+  hdr->active = 0;
+
+  /* after all this it's still possible to have no flags, if you
+   * have no ACL rights */
+  if (*flags && (imap_exec (idata, cmd->data, 0) != 0) &&
+      err_continue && (*err_continue != M_YES))
+  {
+    *err_continue = imap_continue ("imap_sync_message: STORE failed",
+				   idata->cmd.buf);
+    if (*err_continue != M_YES)
+      return -1;
+  }
+
+  hdr->active = 1;
+  idata->ctx->changed--;
+
+  return 0;
+}
+
 /* update the IMAP server to reflect message changes done within mutt.
  * Arguments
  *   ctx: the current context
@@ -932,12 +1007,10 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
   IMAP_DATA* idata;
   CONTEXT* appendctx = NULL;
   BUFFER cmd;
-  char flags[LONG_STRING];
-  char uid[11];
   int deleted;
   int n;
-  int err_continue = M_NO;	/* continue on error? */
   int rc;
+  int err_continue = M_NO;	/* continue on error? */
 
   idata = (IMAP_DATA*) ctx->data;
 
@@ -947,13 +1020,6 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     return -1;
   }
 
-  /* CLOSE purges deleted messages. If we don't want to purge them, we must
-   * tell imap_close_mailbox not to issue the CLOSE command */
-  if (expunge)
-    idata->noclose = 0;
-  else
-    idata->noclose = 1;
-
   /* This function is only called when the calling code	expects the context
    * to be changed. */
   imap_allow_reopen (ctx);
@@ -994,19 +1060,14 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
   {
     if (ctx->hdrs[n]->active && ctx->hdrs[n]->changed)
     {
-      ctx->hdrs[n]->changed = 0;
-
       mutt_message (_("Saving message status flags... [%d/%d]"), n+1,
         ctx->msgcount);
 
-      snprintf (uid, sizeof (uid), "%u", HEADER_DATA(ctx->hdrs[n])->uid);
-      cmd.dptr = cmd.data;
-      mutt_buffer_addstr (&cmd, "UID STORE ");
-      mutt_buffer_addstr (&cmd, uid);
-
-      /* if attachments have been deleted we delete the message and reupload
-       * it. This works better if we're expunging, of course. */
-      if (ctx->hdrs[n]->attach_del)
+      /* if the message has been rethreaded or attachments have been deleted
+       * we delete the message and reupload it.
+       * This works better if we're expunging, of course. */
+      if ((ctx->hdrs[n]->env && (ctx->hdrs[n]->env->refs_changed || ctx->hdrs[n]->env->irt_changed)) ||
+	  ctx->hdrs[n]->attach_del)
       {
 	dprint (3, (debugfile, "imap_sync_mailbox: Attachments to be deleted, falling back to _mutt_save_message\n"));
 	if (!appendctx)
@@ -1018,60 +1079,12 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
 	else
 	  _mutt_save_message (ctx->hdrs[n], appendctx, 1, 0, 0);
       }
-      flags[0] = '\0';
-      
-      imap_set_flag (idata, IMAP_ACL_SEEN, ctx->hdrs[n]->read, "\\Seen ",
-        flags, sizeof (flags));
-      imap_set_flag (idata, IMAP_ACL_WRITE, ctx->hdrs[n]->flagged,
-        "\\Flagged ", flags, sizeof (flags));
-      imap_set_flag (idata, IMAP_ACL_WRITE, ctx->hdrs[n]->replied,
-        "\\Answered ", flags, sizeof (flags));
-      imap_set_flag (idata, IMAP_ACL_DELETE, ctx->hdrs[n]->deleted,
-        "\\Deleted ", flags, sizeof (flags));
 
-      /* now make sure we don't lose custom tags */
-      if (mutt_bit_isset (idata->rights, IMAP_ACL_WRITE))
-        imap_add_keywords (flags, ctx->hdrs[n], idata->flags, sizeof (flags));
-      
-      mutt_remove_trailing_ws (flags);
-      
-      /* UW-IMAP is OK with null flags, Cyrus isn't. The only solution is to
-       * explicitly revoke all system flags (if we have permission) */
-      if (!*flags)
+      if (imap_sync_message (idata, ctx->hdrs[n], &cmd, &err_continue) < 0)
       {
-        imap_set_flag (idata, IMAP_ACL_SEEN, 1, "\\Seen ", flags, sizeof (flags));
-        imap_set_flag (idata, IMAP_ACL_WRITE, 1, "\\Flagged ", flags, sizeof (flags));
-        imap_set_flag (idata, IMAP_ACL_WRITE, 1, "\\Answered ", flags, sizeof (flags));
-        imap_set_flag (idata, IMAP_ACL_DELETE, 1, "\\Deleted ", flags, sizeof (flags));
-
-        mutt_remove_trailing_ws (flags);
-
-	mutt_buffer_addstr (&cmd, " -FLAGS.SILENT (");
+	rc = -1;
+	goto out;
       }
-      else
-	mutt_buffer_addstr (&cmd, " FLAGS.SILENT (");
-      
-      mutt_buffer_addstr (&cmd, flags);
-      mutt_buffer_addstr (&cmd, ")");
-
-      /* dumb hack for bad UW-IMAP 4.7 servers spurious FLAGS updates */
-      ctx->hdrs[n]->active = 0;
-
-      /* after all this it's still possible to have no flags, if you
-       * have no ACL rights */
-      if (*flags && (imap_exec (idata, cmd.data, 0) != 0) &&
-        (err_continue != M_YES))
-      {
-        err_continue = imap_continue ("imap_sync_mailbox: STORE failed",
-          idata->cmd.buf);
-        if (err_continue != M_YES)
-	{
-	  rc = -1;
-	  goto out;
-	}
-      }
-
-      ctx->hdrs[n]->active = 1;
     }
   }
   ctx->changed = 0;
@@ -1091,6 +1104,14 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     }
   }
 
+  if (expunge && ctx->closing)
+  {
+    if (imap_exec (idata, "CLOSE", 0))
+      mutt_error (_("CLOSE failed"));
+  
+    idata->state = IMAP_AUTHENTICATED;
+  }
+
   rc = 0;
  out:
   if (cmd.data)
@@ -1103,7 +1124,7 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
   return rc;
 }
 
-/* imap_close_mailbox: issue close command if neccessary, reset IMAP_DATA */
+/* imap_close_mailbox: clean up IMAP data in CONTEXT */
 void imap_close_mailbox (CONTEXT* ctx)
 {
   IMAP_DATA* idata;
@@ -1114,15 +1135,18 @@ void imap_close_mailbox (CONTEXT* ctx)
   if (!idata)
     return;
 
-  if ((idata->status != IMAP_FATAL) &&
-      (idata->state == IMAP_SELECTED) &&
-      (ctx == idata->ctx))
+  if (ctx == idata->ctx)
   {
-    if (!(idata->noclose) && imap_exec (idata, "CLOSE", 0))
-      mutt_error (_("CLOSE failed"));
+    if (idata->status != IMAP_FATAL && idata->state == IMAP_SELECTED)
+    {
+      /* mx_close_mailbox won't sync if there are no deleted messages
+       * and the mailbox is unchanged, so we may have to close here */
+      if (!ctx->deleted && imap_exec (idata, "CLOSE", 0))
+        mutt_error (_("CLOSE failed"));
+      idata->state = IMAP_AUTHENTICATED;
+    }
 
     idata->reopen &= IMAP_REOPEN_ALLOW;
-    idata->state = IMAP_AUTHENTICATED;
     FREE (&(idata->mailbox));
     mutt_free_list (&idata->flags);
     idata->ctx = NULL;
@@ -1159,7 +1183,7 @@ int imap_check_mailbox (CONTEXT *ctx, int *index_hint, int force)
 
   idata = (IMAP_DATA*) ctx->data;
 
-  if ((force || time(NULL) > idata->lastread + Timeout)
+  if ((force || time(NULL) >= idata->lastread + Timeout)
       && imap_exec (idata, "NOOP", 0) != 0)
     return -1;
 
@@ -1276,6 +1300,151 @@ int imap_mailbox_check (char* path, int new)
   return msgcount;
 }
 
+/* returns number of patterns in the search that should be done server-side
+ * (eg are full-text) */
+static int do_search (const pattern_t* search, int allpats)
+{
+  int rc = 0;
+  const pattern_t* pat;
+
+  for (pat = search; pat; pat = pat->next)
+  {
+    switch (pat->op)
+    {
+      case M_BODY:
+      case M_HEADER:
+      case M_WHOLE_MSG:
+        if (pat->stringmatch)
+          rc++;
+        break;
+      default:
+        if (pat->child && do_search (pat->child, 1))
+          rc++;
+    }
+    
+    if (!allpats)
+      break;
+  }
+  
+  return rc;
+}
+
+/* convert mutt pattern_t to IMAP SEARCH command containing only elements
+ * that require full-text search (mutt already has what it needs for most
+ * match types, and does a better job (eg server doesn't support regexps). */
+static int imap_compile_search (const pattern_t* pat, BUFFER* buf)
+{
+  char term[STRING];
+
+  if (! do_search (pat, 0))
+    return 0;
+
+  if (pat->not)
+    mutt_buffer_addstr (buf, "NOT ");
+
+  if (pat->child)
+  {
+    int clauses;
+
+    if ((clauses = do_search (pat->child, 1)) > 0)
+    {
+      const pattern_t* clause = pat->child;
+
+      mutt_buffer_addch (buf, '(');
+
+      while (clauses)
+      {
+        if (do_search (clause, 0))
+        {
+          if (pat->op == M_OR && clauses > 1)
+            mutt_buffer_addstr (buf, "OR ");
+          clauses--;
+          
+          if (imap_compile_search (clause, buf) < 0)
+            return -1;
+
+          if (clauses)
+            mutt_buffer_addch (buf, ' ');
+          
+          clause = clause->next;
+        }
+      }
+
+      mutt_buffer_addch (buf, ')');
+    }
+  }
+  else
+  {
+    char *delim;
+
+    switch (pat->op)
+    {
+      case M_HEADER:
+        mutt_buffer_addstr (buf, "HEADER ");
+
+        /* extract header name */
+        if (! (delim = strchr (pat->str, ':')))
+        {
+          mutt_error (_("Header search without header name: %s"), pat->str);
+          return -1;
+        }
+        *delim = '\0';
+        imap_quote_string (term, sizeof (term), pat->str);
+        mutt_buffer_addstr (buf, term);
+        mutt_buffer_addch (buf, ' ');
+        
+        /* and field */
+        *delim = ':';
+        delim++;
+        SKIPWS(delim);
+        imap_quote_string (term, sizeof (term), delim);
+        mutt_buffer_addstr (buf, term);
+        break;
+      case M_BODY:
+        mutt_buffer_addstr (buf, "BODY ");
+        imap_quote_string (term, sizeof (term), pat->str);
+        mutt_buffer_addstr (buf, term);
+        break;
+      case M_WHOLE_MSG:
+        mutt_buffer_addstr (buf, "TEXT ");
+        imap_quote_string (term, sizeof (term), pat->str);
+        mutt_buffer_addstr (buf, term);
+        break;
+    }
+  }
+
+  return 0;
+}
+
+int imap_search (CONTEXT* ctx, const pattern_t* pat)
+{
+  BUFFER buf;
+  IMAP_DATA* idata = (IMAP_DATA*)ctx->data;
+  int i;
+
+  for (i = 0; i < ctx->msgcount; i++)
+    ctx->hdrs[i]->matched = 0;
+
+  if (!do_search (pat, 1))
+    return 0;
+
+  memset (&buf, 0, sizeof (buf));
+  mutt_buffer_addstr (&buf, "UID SEARCH ");
+  if (imap_compile_search (pat, &buf) < 0)
+  {
+    FREE (&buf.data);
+    return -1;
+  }
+  if (imap_exec (idata, buf.data, 0) < 0)
+  {
+    FREE (&buf.data);
+    return -1;
+  }
+
+  FREE (&buf.data);
+  return 0;
+}
+
 /* all this listing/browsing is a mess. I don't like that name is a pointer
  *   into idata->buf (used to be a pointer into the passed in buffer, just
  *   as bad), nor do I like the fact that the fetch is done here. This
@@ -1358,6 +1527,8 @@ int imap_subscribe (char *path, int subscribe)
   IMAP_DATA *idata;
   char buf[LONG_STRING];
   char mbox[LONG_STRING];
+  char errstr[STRING];
+  BUFFER err, token;
   IMAP_MBOX mx;
 
   if (!mx_is_imap (path) || imap_parse_path (path, &mx))
@@ -1365,22 +1536,32 @@ int imap_subscribe (char *path, int subscribe)
     mutt_error (_("Bad mailbox name"));
     return -1;
   }
-  
-
   if (!(idata = imap_conn_find (&(mx.account), 0)))
     goto fail;
   
   conn = idata->conn;
 
   imap_fix_path (idata, mx.mbox, buf, sizeof (buf));
+
+  if (option (OPTIMAPCHECKSUBSCRIBED))
+  {
+    memset (&token, 0, sizeof (token));
+    err.data = errstr;
+    err.dsize = sizeof (errstr);
+    snprintf (mbox, sizeof (mbox), "%smailboxes \"%s\"",
+              subscribe ? "" : "un", path);
+    if (mutt_parse_rc_line (mbox, &token, &err))
+      dprint (1, (debugfile, "Error adding subscribed mailbox: %s\n", errstr));
+    FREE (&token.data);
+  }
+  
   if (subscribe)
     mutt_message (_("Subscribing to %s..."), buf);
   else
     mutt_message (_("Unsubscribing to %s..."), buf);
   imap_munge_mbox_name (mbox, sizeof(mbox), buf);
 
-  snprintf (buf, sizeof (buf), "%s %s", subscribe ? "SUBSCRIBE" :
-    "UNSUBSCRIBE", mbox);
+  snprintf (buf, sizeof (buf), "%sSUBSCRIBE %s", subscribe ? "" : "UN", mbox);
 
   if (imap_exec (idata, buf, 0) < 0)
     goto fail;
@@ -1393,6 +1574,73 @@ int imap_subscribe (char *path, int subscribe)
   return -1;
 }
 
+/* trim dest to the length of the longest prefix it shares with src,
+ * returning the length of the trimmed string */
+static int
+longest_common_prefix (char *dest, const char* src, int start, size_t dlen)
+{
+  int pos = start;
+  
+  while (pos < dlen && dest[pos] && dest[pos] == src[pos])
+    pos++;
+  dest[pos] = '\0';
+
+  return pos;
+}
+
+/* look for IMAP URLs to complete from defined mailboxes. Could be extended
+ * to complete over open connections and account/folder hooks too. */
+static int
+imap_complete_hosts (char *dest, size_t len)
+{
+  BUFFY* mailbox;
+  CONNECTION* conn;
+  int rc = -1;
+  int matchlen;
+  
+  matchlen = mutt_strlen (dest);
+  for (mailbox = Incoming; mailbox->next; mailbox = mailbox->next)
+  {
+    if (!mutt_strncmp (dest, mailbox->path, matchlen))
+    {
+      if (rc)
+      {
+        strfcpy (dest, mailbox->path, len);
+        rc = 0;
+      }
+      else
+        longest_common_prefix (dest, mailbox->path, matchlen, len);
+    }
+  }
+  
+  for (conn = mutt_socket_head (); conn->next; conn = conn->next)
+  {
+    ciss_url_t url;
+    char urlstr[LONG_STRING];
+
+    if (conn->account.type != M_ACCT_TYPE_IMAP)
+      continue;
+
+    mutt_account_tourl (&conn->account, &url);
+    /* FIXME: how to handle multiple users on the same host? */
+    url.user = NULL;
+    url.path = NULL;
+    url_ciss_tostring (&url, urlstr, sizeof (urlstr), 0);
+    if (!mutt_strncmp (dest, urlstr, matchlen))
+    {
+      if (rc)
+      {
+        strfcpy (dest, urlstr, len);
+        rc = 0;
+      }
+      else
+        longest_common_prefix (dest, urlstr, matchlen, len);
+    }
+  }
+
+  return rc;
+}
+
 /* imap_complete: given a partial IMAP folder path, return a string which
  *   adds as much to the path as is unique */
 int imap_complete(char* dest, size_t dlen, char* path) {
@@ -1406,19 +1654,22 @@ int imap_complete(char* dest, size_t dlen, char* path) {
   char completion[LONG_STRING];
   int clen, matchlen = 0;
   int completions = 0;
-  int pos = 0;
   IMAP_MBOX mx;
 
-  /* verify passed in path is an IMAP path */
-  if (imap_parse_path (path, &mx))
+  if (imap_parse_path (path, &mx) || !mx.mbox)
   {
-    dprint(2, (debugfile, "imap_complete: bad path %s\n", path));
-    return -1;
+    strfcpy (dest, path, dlen);
+    return imap_complete_hosts (dest, dlen);
   }
 
-  /* don't open a new socket just for completion */
+  /* don't open a new socket just for completion. Instead complete over
+   * known mailboxes/hooks/etc */
   if (!(idata = imap_conn_find (&(mx.account), M_IMAP_CONN_NONEW)))
-    goto fail;
+  {
+    FREE (&mx.mbox);
+    strfcpy (dest, path, dlen);
+    return imap_complete_hosts (dest, dlen);
+  }
   conn = idata->conn;
 
   /* reformat path for IMAP list, and append wildcard */
@@ -1464,13 +1715,7 @@ int imap_complete(char* dest, size_t dlen, char* path) {
         continue;
       }
 
-      pos = 0;
-      while (pos < matchlen && list_word[pos] &&
-          completion[pos] == list_word[pos])
-        pos++;
-      completion[pos] = '\0';
-      matchlen = pos;
-
+      matchlen = longest_common_prefix (completion, list_word, 0, matchlen);
       completions++;
     }
   }
@@ -1485,8 +1730,6 @@ int imap_complete(char* dest, size_t dlen, char* path) {
     FREE (&mx.mbox);
     return 0;
   }
-
- fail:
-  FREE (&mx.mbox);
+  
   return -1;
 }
