@@ -21,6 +21,7 @@
 #define _IMAP_PRIVATE_H 1
 
 #include "imap.h"
+#include "mutt_curses.h"
 #include "mutt_socket.h"
 
 /* -- symbols -- */
@@ -178,9 +179,8 @@ typedef struct
   unsigned char rights[(RIGHTSMAX + 7)/8];
   unsigned int newMailCount;
   IMAP_CACHE cache[IMAP_CACHE_LEN];
-  unsigned int noclose : 1;
 #ifdef USE_HCACHE
-  uint64_t uid_validity;
+  unsigned long uid_validity;
 #endif
   
   /* all folder flags - system flags AND keywords */
@@ -200,9 +200,11 @@ int imap_open_connection (IMAP_DATA* idata);
 IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags);
 int imap_parse_list_response(IMAP_DATA* idata, char** name, int* noselect,
   int* noinferiors, char* delim);
-int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes);
+int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes, progress_t*);
 void imap_expunge_mailbox (IMAP_DATA* idata);
 void imap_logout (IMAP_DATA* idata);
+int imap_sync_message (IMAP_DATA *idata, HEADER *hdr, BUFFER *cmd,
+  int *err_continue);
 
 /* auth.c */
 int imap_authenticate (IMAP_DATA* idata);
