@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2002 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2005 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -15,6 +15,8 @@
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *
+ * $Id: message.c,v 3.27 2005/09/07 07:44:05 brendan Exp $
  */ 
 
 /* message parsing/updating functions */
@@ -27,7 +29,6 @@
 #include <ctype.h>
 
 #include "mutt.h"
-#include "mutt_curses.h"
 #include "imap_private.h"
 #include "message.h"
 #include "mx.h"
@@ -69,16 +70,12 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
 
 #if USE_HCACHE
   void *hc   = NULL;
-  uint64_t *uid_validity = NULL;
+  unsigned long *uid_validity = NULL;
   char uid_buf[64];
 #endif /* USE_HCACHE */
 
   ctx = idata->ctx;
 
-#if USE_HCACHE
-  hc = mutt_hcache_open (HeaderCache, ctx->path);
-#endif /* USE_HCACHE */
-
   if (mutt_bit_isset (idata->capabilities,IMAP4REV1))
   {
     snprintf (hdrreq, sizeof (hdrreq), "BODY.PEEK[HEADER.FIELDS (%s%s%s)]", 
@@ -93,9 +90,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   {	/* Unable to fetch headers for lower versions */
     mutt_error _("Unable to fetch headers from this IMAP server version.");
     mutt_sleep (2);	/* pause a moment to let the user see the error */
-#if USE_HCACHE
-    mutt_hcache_close (hc);
-#endif /* USE_HCACHE */
     return -1;
   }
 
@@ -106,9 +100,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   {
     mutt_error (_("Could not create temporary file %s"), tempfile);
     mutt_sleep (2);
-#if USE_HCACHE
-    mutt_hcache_close (hc);
-#endif /* USE_HCACHE */
     return -1;
   }
   unlink (tempfile);
@@ -122,6 +113,8 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   idata->newMailCount = 0;
 
 #if USE_HCACHE
+  hc = mutt_hcache_open (HeaderCache, ctx->path);
+
   snprintf (buf, sizeof (buf),
     "FETCH %d:%d (UID FLAGS)", msgbegin + 1, msgend + 1);
   fetchlast = msgend + 1;
@@ -154,27 +147,27 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       fputs ("\n\n", fp);
 
       sprintf(uid_buf, "/%u", h.data->uid); /* XXX --tg 21:41 04-07-11 */
-      uid_validity = (uint64_t *) mutt_hcache_fetch (hc, uid_buf, &imap_hcache_keylen);
+      uid_validity = (unsigned long *) mutt_hcache_fetch (hc, uid_buf, &imap_hcache_keylen);
 
-      if (uid_validity != NULL
-      && *uid_validity == idata->uid_validity) {
-	      ctx->hdrs[msgno] = mutt_hcache_restore((unsigned char *) uid_validity, 0);
-	      ctx->hdrs[msgno]->index = h.sid - 1;
-	      if (h.sid != ctx->msgcount + 1)
-		      dprint (1, (debugfile, "imap_read_headers: msgcount and sequence ID are inconsistent!"));
-	      /* messages which have not been expunged are ACTIVE (borrowed from mh 
-	       * folders) */
-	      ctx->hdrs[msgno]->active = 1;
-	      ctx->hdrs[msgno]->read = h.read;
-	      ctx->hdrs[msgno]->old = h.old;
-	      ctx->hdrs[msgno]->deleted = h.deleted;
-	      ctx->hdrs[msgno]->flagged = h.flagged;
-	      ctx->hdrs[msgno]->replied = h.replied;
-	      ctx->hdrs[msgno]->changed = h.changed;
-	      /*  ctx->hdrs[msgno]->received is restored from mutt_hcache_restore */
-	      ctx->hdrs[msgno]->data = (void *) (h.data);
-
-	      ctx->msgcount++;
+      if (uid_validity != NULL && *uid_validity == idata->uid_validity)
+      {
+	ctx->hdrs[msgno] = mutt_hcache_restore((unsigned char *) uid_validity, 0);
+	ctx->hdrs[msgno]->index = h.sid - 1;
+	if (h.sid != ctx->msgcount + 1)
+	  dprint (1, (debugfile, "imap_read_headers: msgcount and sequence ID are inconsistent!"));
+	/* messages which have not been expunged are ACTIVE (borrowed from mh 
+	 * folders) */
+	ctx->hdrs[msgno]->active = 1;
+	ctx->hdrs[msgno]->read = h.read;
+        ctx->hdrs[msgno]->old = h.old;
+        ctx->hdrs[msgno]->deleted = h.deleted;
+        ctx->hdrs[msgno]->flagged = h.flagged;
+        ctx->hdrs[msgno]->replied = h.replied;
+        ctx->hdrs[msgno]->changed = h.changed;
+        /*  ctx->hdrs[msgno]->received is restored from mutt_hcache_restore */
+        ctx->hdrs[msgno]->data = (void *) (h.data);
+        
+        ctx->msgcount++;
       }
       rewind (fp);
 
@@ -188,7 +181,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     {
       imap_free_header_data ((void**) &h.data);
       fclose (fp);
-  mutt_hcache_close (hc);
+      mutt_hcache_close (hc);
       return -1;
     }
   }
@@ -291,7 +284,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       imap_free_header_data ((void**) &h.data);
       fclose (fp);
 #if USE_HCACHE
-  mutt_hcache_close (hc);
+      mutt_hcache_close (hc);
 #endif /* USE_HCACHE */
       return -1;
     }
@@ -314,7 +307,10 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   fclose(fp);
 
   if (ctx->msgcount > oldmsgcount)
+  {
+    mx_alloc_memory(ctx);
     mx_update_context (ctx, ctx->msgcount - oldmsgcount);
+  }
 
   return msgend;
 }
@@ -323,10 +319,12 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
 {
   IMAP_DATA* idata;
   HEADER* h;
+  ENVELOPE* newenv;
   char buf[LONG_STRING];
   char path[_POSIX_PATH_MAX];
   char *pc;
   long bytes;
+  progress_t progressbar;
   int uid;
   int cacheno;
   IMAP_CACHE *cache;
@@ -345,19 +343,12 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
 
   if (cache->path)
   {
-    if (cache->uid == HEADER_DATA(h)->uid)
-    {
-      /* yes, so just return a pointer to the message */
-      if (!(msg->fp = fopen (cache->path, "r")))
-      {
-	mutt_perror (cache->path);
-	return (-1);
-      }
+    /* don't treat cache errors as fatal, just fall back. */
+    if (cache->uid == HEADER_DATA(h)->uid &&
+        (msg->fp = fopen (cache->path, "r")))
       return 0;
-    }
     else
     {
-      /* clear the previous entry */
       unlink (cache->path);
       FREE (&cache->path);
     }
@@ -418,7 +409,10 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
 	    imap_error ("imap_fetch_message()", buf);
 	    goto bail;
 	  }
-	  if (imap_read_literal (msg->fp, idata, bytes) < 0)
+          progressbar.size = bytes;
+          progressbar.msg = _("Fetching message...");
+          mutt_progress_bar (&progressbar, 0);
+	  if (imap_read_literal (msg->fp, idata, bytes, &progressbar) < 0)
 	    goto bail;
 	  /* pick up trailing line */
 	  if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
@@ -444,6 +438,13 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   /* see comment before command start. */
   h->active = 1;
 
+  fflush (msg->fp);
+  if (ferror (msg->fp))
+  {
+    mutt_perror (cache->path);
+    goto bail;
+  }
+  
   if (rc != IMAP_CMD_OK)
     goto bail;
 
@@ -460,21 +461,8 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
    * picked up in mutt_read_rfc822_header, we mark the message (and context
    * changed). Another possiblity: ignore Status on IMAP?*/
   read = h->read;
-  /* I hate do this here, since it's so low-level, but I'm not sure where
-   * I can abstract it. Problem: the id and subj hashes lose their keys when
-   * mutt_free_envelope gets called, but keep their spots in the hash. This
-   * confuses threading. Alternatively we could try to merge the new
-   * envelope into the old one. Also messy and lowlevel. */
-  if (ctx->id_hash && h->env->message_id)
-    hash_delete (ctx->id_hash, h->env->message_id, h, NULL);
-  if (ctx->subj_hash && h->env->real_subj)
-    hash_delete (ctx->subj_hash, h->env->real_subj, h, NULL);
-  mutt_free_envelope (&h->env);
-  h->env = mutt_read_rfc822_header (msg->fp, h, 0, 0);
-  if (ctx->id_hash && h->env->message_id)
-    hash_insert (ctx->id_hash, h->env->message_id, h, 0);
-  if (ctx->subj_hash && h->env->real_subj)
-    hash_insert (ctx->subj_hash, h->env->real_subj, h, 1);
+  newenv = mutt_read_rfc822_header (msg->fp, h, 0, 0);
+  mutt_merge_envelopes(h->env, &newenv);
 
   /* see above. We want the new status in h->read, so we unset it manually
    * and let mutt_set_flag set it correctly, updating context. */
@@ -523,6 +511,8 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
   char mbox[LONG_STRING];
   char mailbox[LONG_STRING]; 
   size_t len;
+  progress_t progressbar;
+  size_t sent;
   int c, last;
   IMAP_MBOX mx;
   int rc;
@@ -554,6 +544,10 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     len++;
   }
   rewind (fp);
+
+  progressbar.msg = _("Uploading message...");
+  progressbar.size = len;
+  mutt_progress_bar (&progressbar, 0);
   
   imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
   snprintf (buf, sizeof (buf), "APPEND %s (%s%s%s%s%s) {%lu}", mbox,
@@ -586,9 +580,7 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     goto fail;
   }
 
-  mutt_message _("Uploading message ...");
-
-  for (last = EOF, len = 0; (c = fgetc(fp)) != EOF; last = c)
+  for (last = EOF, sent = len = 0; (c = fgetc(fp)) != EOF; last = c)
   {
     if (c == '\n' && last != '\r')
       buf[len++] = '\r';
@@ -596,7 +588,11 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     buf[len++] = c;
 
     if (len > sizeof(buf) - 3)
+    {
+      sent += len;
       flush_buffer(buf, &len, idata->conn);
+      mutt_progress_bar (&progressbar, sent);
+    }
   }
   
   if (len)
@@ -640,13 +636,14 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
 int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
 {
   IMAP_DATA* idata;
-  BUFFER cmd;
+  BUFFER cmd, sync_cmd;
   char uid[11];
   char mbox[LONG_STRING];
   char mmbox[LONG_STRING];
   int rc;
   int n;
   IMAP_MBOX mx;
+  int err_continue = M_NO;
 
   idata = (IMAP_DATA*) ctx->data;
 
@@ -672,6 +669,7 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
   
   imap_fix_path (idata, mx.mbox, mbox, sizeof (mbox));
 
+  memset (&sync_cmd, 0, sizeof (sync_cmd));
   memset (&cmd, 0, sizeof (cmd));
   mutt_buffer_addstr (&cmd, "UID COPY ");
 
@@ -688,6 +686,17 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
 	dprint (3, (debugfile, "imap_copy_messages: Message contains attachments to be deleted\n"));
 	return 1;
       }
+
+      if (ctx->hdrs[n]->tagged && ctx->hdrs[n]->active &&
+	  ctx->hdrs[n]->changed)
+      {
+	rc = imap_sync_message (idata, ctx->hdrs[n], &sync_cmd, &err_continue);
+	if (rc < 0)
+	{
+	  dprint (1, (debugfile, "imap_copy_messages: could not sync\n"));
+	  goto fail;
+	}
+      }
     }
 
     rc = imap_make_msg_set (idata, &cmd, M_TAG, 0);
@@ -703,6 +712,16 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
     mutt_message (_("Copying message %d to %s..."), h->index+1, mbox);
     snprintf (uid, sizeof (uid), "%u", HEADER_DATA (h)->uid);
     mutt_buffer_addstr (&cmd, uid);
+
+    if (h->active && h->changed)
+    {
+      rc = imap_sync_message (idata, h, &sync_cmd, &err_continue);
+      if (rc < 0)
+      {
+	dprint (1, (debugfile, "imap_copy_messages: could not sync\n"));
+	goto fail;
+      }
+    }
   }
 
   /* let's get it on */
@@ -761,12 +780,16 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
 
   if (cmd.data)
     FREE (&cmd.data);
+  if (sync_cmd.data)
+    FREE (&sync_cmd.data);
   FREE (&mx.mbox);
   return 0;
 
  fail:
   if (cmd.data)
     FREE (&cmd.data);
+  if (sync_cmd.data)
+    FREE (&sync_cmd.data);
   FREE (&mx.mbox);
   return -1;
 }
@@ -886,7 +909,7 @@ static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf, FILE* fp)
   
   if (imap_get_literal_count (buf, &bytes) == 0)
   {
-    imap_read_literal (fp, idata, bytes);
+    imap_read_literal (fp, idata, bytes, NULL);
 
     /* we may have other fields of the FETCH _after_ the literal
      * (eg Domino puts FLAGS here). Nothing wrong with that, either.
@@ -923,7 +946,6 @@ static size_t imap_hcache_keylen (const char *fn)
 static int msg_fetch_header_fetch (CONTEXT* ctx, IMAP_HEADER* h, char* buf, FILE* fp)
 {
   IMAP_DATA* idata;
-  long bytes;
   int rc = -1; /* default now is that string isn't FETCH response*/
 
   idata = (IMAP_DATA*) ctx->data;
