@@ -31,7 +31,7 @@
 #include "mutt_crypt.h"
 #include "mutt_idna.h"
 
-#if defined(USE_SSL) || defined(USE_NSS) || defined(USE_GNUTLS)
+#if defined(USE_SSL) || defined(USE_GNUTLS)
 #include "mutt_ssl.h"
 #endif
 
@@ -123,7 +123,7 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
 	  (ch == '#' && !(flags & M_TOKEN_COMMENT)) ||
 	  (ch == '=' && (flags & M_TOKEN_EQUAL)) ||
 	  (ch == ';' && !(flags & M_TOKEN_SEMICOLON)) ||
-	  ((flags & M_TOKEN_PATTERN) && strchr ("~!|", ch)))
+	  ((flags & M_TOKEN_PATTERN) && strchr ("~=!|", ch)))
 	break;
     }
 
@@ -272,7 +272,7 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
       }
       else
       {
-	for (pc = tok->dptr; isalpha ((unsigned char) *pc) || *pc == '_'; pc++)
+	for (pc = tok->dptr; isalnum ((unsigned char) *pc) || *pc == '_'; pc++)
 	  ;
 	var = mutt_substrdup (tok->dptr, pc);
 	tok->dptr = pc;
@@ -289,6 +289,48 @@ int mutt_extract_token (BUFFER *dest, BUFFER *tok, int flags)
   return 0;
 }
 
+static void mutt_free_opt (struct option_t* p)
+{
+  REGEXP* pp;
+
+  switch (p->type & DT_MASK)
+  {
+  case DT_ADDR:
+    rfc822_free_address ((ADDRESS**)p->data);
+    break;
+  case DT_RX:
+    pp = (REGEXP*)p->data;
+    FREE (&pp->pattern);
+    if (pp->rx)
+    {
+      regfree (pp->rx);
+      FREE (&pp->rx);
+    }
+    break;
+  case DT_PATH:
+  case DT_STR:
+    FREE ((char**)p->data);
+    break;
+  }
+}
+
+/* clean up before quitting */
+void mutt_free_opts (void)
+{
+  int i;
+
+  for (i = 0; MuttVars[i].option; i++)
+    mutt_free_opt (MuttVars + i);
+
+  mutt_free_rx_list (&Alternates);
+  mutt_free_rx_list (&UnAlternates);
+  mutt_free_rx_list (&MailLists);
+  mutt_free_rx_list (&UnMailLists);
+  mutt_free_rx_list (&SubscribedLists);
+  mutt_free_rx_list (&UnSubscribedLists);
+  mutt_free_rx_list (&NoSpamList);
+}
+
 static void add_to_list (LIST **list, const char *str)
 {
   LIST *t, *last = NULL;
@@ -448,6 +490,8 @@ static int remove_from_spam_list (SPAM_LIST **list, const char *pat)
 
   /* Being first is a special case. */
   spam = *list;
+  if (!spam)
+    return 0;
   if (spam->rx && !mutt_strcmp(spam->rx->pattern, pat))
   {
     *list = spam->next;
@@ -585,42 +629,6 @@ static int parse_list (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   return 0;
 }
 
-static int _parse_rx_list (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err, int flags)
-{
-  do 
-  {
-    mutt_extract_token (buf, s, 0);
-    if (add_to_rx_list ((RX_LIST **) data, buf->data, flags, err) != 0)
-      return -1;
-	
-  }
-  while (MoreArgs (s));
-  
-  return 0;
-}
-
-static int parse_rx_list (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
-{
-  return _parse_rx_list (buf, s, data, err, REG_ICASE);
-}
-
-static int parse_rx_unlist (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
-{
-  do
-  {
-    mutt_extract_token (buf, s, 0);
-    if (mutt_strcmp (buf->data, "*") == 0)
-    {
-      mutt_free_rx_list ((RX_LIST **) data);
-      break;
-    }
-    remove_from_rx_list ((RX_LIST **) data, buf->data);
-  }
-  while (MoreArgs (s));
-  
-  return 0;
-}
-
 static void _alternates_clean (void)
 {
   int i;
@@ -2042,6 +2050,29 @@ int mutt_var_value_complete (char *buffer, size_t len, int pos)
 		  (*((short *) MuttVars[idx].data) & SORT_LAST) ? "last-" : "",
 		  p);
       }
+      else if (DTYPE (MuttVars[idx].type) == DT_MAGIC)
+      {
+        char *p;
+
+        switch (DefaultMagic)
+        {
+          case M_MBOX:
+            p = "mbox";
+            break;
+          case M_MMDF:
+            p = "MMDF";
+            break;
+          case M_MH:
+            p = "MH";
+            break;
+          case M_MAILDIR:
+            p = "Maildir";
+            break;
+          default:
+            p = "unknown";
+        }
+        strfcpy (tmp, p, sizeof (tmp));
+      }
       else if (DTYPE (MuttVars[idx].type) == DT_BOOL)
 	strfcpy (tmp, option (MuttVars[idx].data) ? "yes" : "no", sizeof (tmp));
       else
@@ -2096,6 +2127,38 @@ int mutt_query_variables (LIST *queries)
   return 0;
 }
 
+/* dump out the value of all the variables we have */
+int mutt_dump_variables (void)
+{
+  int i;
+  
+  char errbuff[STRING];
+  char command[STRING];
+  
+  BUFFER err, token;
+  
+  memset (&err, 0, sizeof (err));
+  memset (&token, 0, sizeof (token));
+  
+  err.data = errbuff;
+  err.dsize = sizeof (errbuff);
+  
+  for (i = 0; MuttVars[i].option; i++)
+  {
+    snprintf (command, sizeof (command), "set ?%s\n", MuttVars[i].option);
+    if (mutt_parse_rc_line (command, &token, &err) == -1)
+    {
+      fprintf (stderr, "%s\n", err.data);
+      FREE (&token.data);
+      return 1;
+    }
+    printf("%s\n", err.data);
+  }
+  
+  FREE (&token.data);
+  return 0;
+}
+
 char *mutt_getnamebyvalue (int val, const struct mapping_t *map)
 {
   int i;
@@ -2197,6 +2260,7 @@ void mutt_init (int skip_sys_rc, LIST *commands)
 
     Realname = safe_strdup (mutt_gecos_name (rnbuf, sizeof (rnbuf), pw));
     Shell = safe_strdup (pw->pw_shell);
+    endpwent ();
   }
   else 
   {
