@@ -83,6 +83,9 @@ static struct mapping_t KeyNames[] = {
 #ifdef KEY_BTAB
   { "<BackTab>", KEY_BTAB },
 #endif
+#ifdef KEY_NEXT
+  { "<Next>",    KEY_NEXT },
+#endif  
   { NULL,	0 }
 };
 
@@ -341,7 +344,7 @@ static void push_string (char *s)
 	}
       }
     }
-    mutt_ungetch (*p--, 0);
+    mutt_ungetch ((unsigned char)*p--, 0);	/* independent 8 bits chars */
   }
 }
 
