@@ -25,8 +25,6 @@
 #ifndef _LIB_H
 # define _LIB_H
 
-# include "config.h"
-
 # include <stdio.h>
 # include <string.h>
 # ifdef HAVE_UNISTD_H
@@ -105,6 +103,10 @@ extern void (*mutt_error) (const char *, ...);
 # endif
 void mutt_exit (int);
 
+/* Exit values used in send_msg() */
+#define S_ERR 127
+#define S_BKG 126
+
 /* The actual library functions. */
 
 FILE *safe_fopen (const char *, const char *);
@@ -115,6 +117,8 @@ char *mutt_skip_whitespace (char *);
 char *mutt_strlower (char *);
 char *mutt_substrcpy (char *, const char *, const char *, size_t);
 char *mutt_substrdup (const char *, const char *);
+char *safe_strcat (char *, size_t, const char *);
+char *safe_strncat (char *, size_t, const char *, size_t);
 char *safe_strdup (const char *);
 
 const char *mutt_stristr (const char *, const char *);
@@ -147,4 +151,5 @@ void mutt_unlink (const char *);
 void safe_free (void *);
 void safe_realloc (void *, size_t);
 
+const char *mutt_strsysexit(int e);
 #endif
