@@ -32,6 +32,14 @@
 #include "mutt_crypt.h"
 #include "mutt_idna.h"
 
+#ifdef USE_SASL
+#include "mutt_sasl.h"
+#endif
+
+#ifdef USE_IMAP
+#include "imap/imap.h"
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <locale.h>
@@ -59,11 +67,11 @@ Mutt is free software, and you are welcome to redistribute it\n\
 under certain conditions; type `mutt -vv' for details.\n");
 
 static const char *Copyright = N_("\
-Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>\n\
+Copyright (C) 1996-2004 Michael R. Elkins <me@mutt.org>\n\
 Copyright (C) 1996-2002 Brandon Long <blong@fiction.net>\n\
-Copyright (C) 1997-2002 Thomas Roessler <roessler@does-not-exist.org>\n\
-Copyright (C) 1998-2002 Werner Koch <wk@isil.d.shuttle.de>\n\
-Copyright (C) 1999-2002 Brendan Cully <brendan@kublai.com>\n\
+Copyright (C) 1997-2005 Thomas Roessler <roessler@does-not-exist.org>\n\
+Copyright (C) 1998-2005 Werner Koch <wk@isil.d.shuttle.de>\n\
+Copyright (C) 1999-2005 Brendan Cully <brendan@kublai.com>\n\
 Copyright (C) 1999-2002 Tommi Komulainen <Tommi.Komulainen@iki.fi>\n\
 Copyright (C) 2000-2002 Edmund Grimley Evans <edmundo@rano.org>\n\
 \n\
@@ -99,6 +107,7 @@ static void mutt_usage (void)
 "usage: mutt [ -nRyzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> ]\n\
        mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -Q <query> [ -Q <query> ] [...]\n\
        mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -A <alias> [ -A <alias> ] [...]\n\
+       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -D\n\
        mutt [ -nx ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
        mutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
        mutt -v[v]\n\
@@ -108,7 +117,12 @@ options:\n\
   -a <file>\tattach a file to the message\n\
   -b <address>\tspecify a blind carbon-copy (BCC) address\n\
   -c <address>\tspecify a carbon-copy (CC) address\n\
-  -e <command>\tspecify a command to be executed after initialization\n\
+  -D\t\tprint the value of all variables to stdout");
+#if DEBUG
+  puts _("  -d <level>\tlog debugging output to ~/.muttdebug0");
+#endif
+  puts _(
+"  -e <command>\tspecify a command to be executed after initialization\n\
   -f <file>\tspecify which mailbox to read\n\
   -F <file>\tspecify an alternate muttrc file\n\
   -H <file>\tspecify a draft file to read header and body from\n\
@@ -262,11 +276,6 @@ static void show_version (void)
 	"+USE_SASL  "
 #else
 	"-USE_SASL  "
-#endif
-#ifdef USE_SASL2
-	"+USE_SASL2  "
-#else
-	"-USE_SASL2  "
 #endif
 	"\n"
 	
@@ -517,6 +526,7 @@ int main (int argc, char **argv)
   int version = 0;
   int i;
   int explicit_folder = 0;
+  int dump_variables = 0;
   extern char *optarg;
   extern int optind;
 
@@ -546,7 +556,7 @@ int main (int argc, char **argv)
   memset (Options, 0, sizeof (Options));
   memset (QuadOptions, 0, sizeof (QuadOptions));
   
-  while ((i = getopt (argc, argv, "A:a:b:F:f:c:d:e:H:s:i:hm:npQ:RvxyzZ")) != EOF)
+  while ((i = getopt (argc, argv, "A:a:b:F:f:c:Dd:e:H:s:i:hm:npQ:RvxyzZ")) != EOF)
     switch (i)
     {
       case 'A':
@@ -577,6 +587,10 @@ int main (int argc, char **argv)
 	  msg->env->cc = rfc822_parse_adrlist (msg->env->cc, optarg);
 	break;
 
+      case 'D':
+	dump_variables = 1;
+	break;
+
       case 'd':
 #ifdef DEBUG
 	debuglevel = atoi (optarg);
@@ -662,7 +676,7 @@ int main (int argc, char **argv)
   }
 
   /* Check for a batch send. */
-  if (!isatty (0) || queries || alias_queries)
+  if (!isatty (0) || queries || alias_queries || dump_variables)
   {
     set_option (OPTNOCURSES);
     sendflags = SENDBATCH;
@@ -682,6 +696,8 @@ int main (int argc, char **argv)
 
   if (queries)
     return mutt_query_variables (queries);
+  if (dump_variables)
+    return mutt_dump_variables();
 
   if (alias_queries)
   {
@@ -910,6 +926,7 @@ int main (int argc, char **argv)
       strfcpy (folder, NONULL(Spoolfile), sizeof (folder));
     mutt_expand_path (folder, sizeof (folder));
 
+    mutt_str_replace (&CurrentFolder, folder);
     mutt_str_replace (&LastFolder, folder);
 
     if (flags & M_IGNORE)
@@ -935,6 +952,13 @@ int main (int argc, char **argv)
       if (Context)
 	FREE (&Context);
     }
+#ifdef USE_IMAP
+    imap_logout_all ();
+#endif
+#ifdef USE_SASL
+    mutt_sasl_done ();
+#endif
+    mutt_free_opts ();
     mutt_endwin (Errorbuf);
   }
 
