@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,9 @@
  * This file is named mutt_menu.h so it doesn't collide with ncurses menu.h
  */
 
+#ifndef _MUTT_MENU_H_
+#define _MUTT_MENU_H_ 1
+
 #include "keymap.h"
 #include "mutt_regex.h"
 
@@ -62,7 +65,7 @@ typedef struct menu_t
   /* how to search the menu */
   int (*search) (struct menu_t *, regex_t *re, int n);
 
-  int (*tag) (struct menu_t *, int i);
+  int (*tag) (struct menu_t *, int i, int m);
 
   /* color pair to be used for the requested element 
    * (default function returns ColorDefs[MT_COLOR_NORMAL])
@@ -83,6 +86,7 @@ void menu_redraw_index (MUTTMENU *);
 void menu_redraw_status (MUTTMENU *);
 void menu_redraw_motion (MUTTMENU *);
 void menu_redraw_current (MUTTMENU *);
+int  menu_redraw (MUTTMENU *);
 void menu_first_entry (MUTTMENU *);
 void menu_last_entry (MUTTMENU *);
 void menu_top_page (MUTTMENU *);
@@ -107,3 +111,5 @@ int mutt_menuLoop (MUTTMENU *);
 /* used in both the index and pager index to make an entry. */
 void index_make_entry (char *, size_t, struct menu_t *, int);
 int index_color (int);
+
+#endif /* _MUTT_MENU_H_ */
