@@ -27,17 +27,12 @@
 #include "mutt_sasl.h"
 #include "mutt_socket.h"
 
-#ifdef USE_SASL2
 #include <errno.h>
 #include <netdb.h>
 #include <sasl/sasl.h>
-#else
-#include <sasl.h>
-#endif
 #include <sys/socket.h>
 #include <netinet/in.h>
 
-#ifdef USE_SASL2
 static int getnameinfo_err(int ret)
 {
   int err;
@@ -79,16 +74,13 @@ static int getnameinfo_err(int ret)
   }
   return err;
 }
-#endif
 
 /* arbitrary. SASL will probably use a smaller buffer anyway. OTOH it's
  * been a while since I've had access to an SASL server which negotiated
  * a protection buffer. */ 
 #define M_SASL_MAXBUF 65536
 
-#ifdef USE_SASL2
 #define IP_PORT_BUFLEN 1024
-#endif
 
 static sasl_callback_t mutt_sasl_callbacks[5];
 
@@ -108,7 +100,6 @@ static int mutt_sasl_conn_read (CONNECTION* conn, char* buf, size_t len);
 static int mutt_sasl_conn_write (CONNECTION* conn, const char* buf,
   size_t count);
 
-#ifdef USE_SASL2
 /* utility function, stolen from sasl2 sample code */
 static int iptostring(const struct sockaddr *addr, socklen_t addrlen,
                      char *out, unsigned outlen) {
@@ -133,10 +124,9 @@ static int iptostring(const struct sockaddr *addr, socklen_t addrlen,
 
     return SASL_OK;
 }
-#endif
 
 /* mutt_sasl_start: called before doing a SASL exchange - initialises library
- *   (if neccessary). */
+ *   (if necessary). */
 int mutt_sasl_start (void)
 {
   static unsigned char sasl_init = 0;
@@ -175,13 +165,9 @@ int mutt_sasl_start (void)
 int mutt_sasl_client_new (CONNECTION* conn, sasl_conn_t** saslconn)
 {
   sasl_security_properties_t secprops;
-#ifdef USE_SASL2
   struct sockaddr_storage local, remote;
   socklen_t size;
   char iplocalport[IP_PORT_BUFLEN], ipremoteport[IP_PORT_BUFLEN];
-#else
-  sasl_external_properties_t extprops;
-#endif
   const char* service;
   int rc;
 
@@ -201,7 +187,6 @@ int mutt_sasl_client_new (CONNECTION* conn, sasl_conn_t** saslconn)
       return -1;
   }
 
-#ifdef USE_SASL2
   size = sizeof (local);
   if (getsockname (conn->fd, (struct sockaddr *)&local, &size)){
     dprint (1, (debugfile, "mutt_sasl_client_new: getsockname for local failed\n"));
@@ -224,16 +209,11 @@ int mutt_sasl_client_new (CONNECTION* conn, sasl_conn_t** saslconn)
     return -1;
   }
 
-dprint(1,(debugfile, "local ip: %s, remote ip:%s\n", iplocalport, ipremoteport));
+  dprint(1,(debugfile, "local ip: %s, remote ip:%s\n", iplocalport, ipremoteport));
   
   rc = sasl_client_new (service, conn->account.host, iplocalport, ipremoteport,
     mutt_sasl_get_callbacks (&conn->account), 0, saslconn);
 
-#else
-  rc = sasl_client_new (service, conn->account.host,
-    mutt_sasl_get_callbacks (&conn->account), SASL_SECURITY_LAYER, saslconn);
-#endif
-
   if (rc != SASL_OK)
   {
     dprint (1, (debugfile,
@@ -241,43 +221,6 @@ dprint(1,(debugfile, "local ip: %s, remote ip:%s\n", iplocalport, ipremoteport))
     return -1;
   }
 
-  /*** set sasl IP properties, necessary for use with krb4 ***/
-  /* Do we need to fail if this fails? I would assume having these unset
-   * would just disable KRB4. Who wrote this code? I'm not sure how this
-   * interacts with the NSS code either, since that mucks with the fd. */
-#ifndef USE_SASL2 /* with SASLv2 this all happens in sasl_client_new */
-  {
-    struct sockaddr_in local, remote;
-    socklen_t size;
-
-    size = sizeof (local);
-    if (getsockname (conn->fd, (struct sockaddr*) &local, &size))
-      return -1;
-
-    size = sizeof(remote);
-    if (getpeername(conn->fd, (struct sockaddr*) &remote, &size))
-      return -1;
-
-#ifdef SASL_IP_LOCAL
-    if (sasl_setprop(*saslconn, SASL_IP_LOCAL, &local) != SASL_OK)
-    {
-      dprint (1, (debugfile,
-	"mutt_sasl_client_new: Error setting local IP address\n"));
-      return -1;
-    }
-#endif
-
-#ifdef SASL_IP_REMOTE
-    if (sasl_setprop(*saslconn, SASL_IP_REMOTE, &remote) != SASL_OK)
-    {
-      dprint (1, (debugfile,
-	"mutt_sasl_client_new: Error setting remote IP address\n"));
-      return -1;
-    }
-#endif
-  }
-#endif
-
   /* set security properties. We use NOPLAINTEXT globally, since we can
    * just fall back to LOGIN in the IMAP case anyway. If that doesn't
    * work for POP, we can make it a flag or move this code into
@@ -294,36 +237,22 @@ dprint(1,(debugfile, "local ip: %s, remote ip:%s\n", iplocalport, ipremoteport))
     return -1;
   }
 
-  /* we currently don't have an SSF finder for NSS (I don't know the API).
-   * If someone does it'd probably be trivial to write mutt_nss_get_ssf().
-   * I have a feeling more SSL code could be shared between those two files,
-   * but I haven't looked into it yet, since I still don't know the APIs. */
-#if defined(USE_SSL) || defined(USE_GNUTLS)
   if (conn->ssf)
   {
-#ifdef USE_SASL2 /* I'm not sure this actually has an effect, at least with SASLv2 */
+    /* I'm not sure this actually has an effect, at least with SASLv2 */
     dprint (2, (debugfile, "External SSF: %d\n", conn->ssf));
     if (sasl_setprop (*saslconn, SASL_SSF_EXTERNAL, &(conn->ssf)) != SASL_OK)
-#else
-    memset (&extprops, 0, sizeof (extprops));
-    extprops.ssf = conn->ssf;
-    dprint (2, (debugfile, "External SSF: %d\n", extprops.ssf));
-    if (sasl_setprop (*saslconn, SASL_SSF_EXTERNAL, &extprops) != SASL_OK)
-#endif
     {
       dprint (1, (debugfile, "mutt_sasl_client_new: Error setting external properties\n"));
       return -1;
     }
-#ifdef USE_SASL2
     dprint (2, (debugfile, "External authentication name: %s\n", conn->account.user));
     if (sasl_setprop (*saslconn, SASL_AUTH_EXTERNAL, conn->account.user) != SASL_OK)
      {
       dprint (1, (debugfile, "mutt_sasl_client_new: Error setting external properties\n"));
       return -1;
     }
-#endif
   }
-#endif
 
   return 0;
 }
@@ -334,12 +263,12 @@ sasl_callback_t* mutt_sasl_get_callbacks (ACCOUNT* account)
 
   callback = mutt_sasl_callbacks;
 
-  callback->id = SASL_CB_AUTHNAME;
+  callback->id = SASL_CB_USER;
   callback->proc = mutt_sasl_cb_authname;
   callback->context = account;
   callback++;
 
-  callback->id = SASL_CB_USER;
+  callback->id = SASL_CB_AUTHNAME;
   callback->proc = mutt_sasl_cb_authname;
   callback->context = account;
   callback++;
@@ -377,7 +306,7 @@ int mutt_sasl_interact (sasl_interact_t* interaction)
 
     interaction->len = mutt_strlen (resp)+1;
     interaction->result = safe_malloc (interaction->len);
-    memcpy (interaction->result, resp, interaction->len);
+    memcpy ((char *)interaction->result, resp, interaction->len);
 
     interaction++;
   }
@@ -407,19 +336,11 @@ void mutt_sasl_setup_conn (CONNECTION* conn, sasl_conn_t* saslconn)
 
   sasldata->saslconn = saslconn;
   /* get ssf so we know whether we have to (en|de)code read/write */
-#ifdef USE_SASL2
   sasl_getprop (saslconn, SASL_SSF, (const void**) &sasldata->ssf);
-#else
-  sasl_getprop (saslconn, SASL_SSF, (void**) &sasldata->ssf);
-#endif
   dprint (3, (debugfile, "SASL protection strength: %u\n", *sasldata->ssf));
   /* Add SASL SSF to transport SSF */
   conn->ssf += *sasldata->ssf;
-#ifdef USE_SASL2
   sasl_getprop (saslconn, SASL_MAXOUTBUF, (const void**) &sasldata->pbufsize);
-#else
-  sasl_getprop (saslconn, SASL_MAXOUTBUF, (void**) &sasldata->pbufsize);
-#endif
   dprint (3, (debugfile, "SASL protection buffer size: %u\n", *sasldata->pbufsize));
 
   /* clear input buffer */
@@ -450,8 +371,12 @@ static int mutt_sasl_cb_log (void* context, int priority, const char* message)
   return SASL_OK;
 }
 
-/* mutt_sasl_cb_authname: callback to retrieve authname or user (mutt
- *   doesn't distinguish, even if some SASL plugins do) from ACCOUNT */
+void mutt_sasl_done (void)
+{
+  sasl_done ();
+}
+
+/* mutt_sasl_cb_authname: callback to retrieve authname or user from ACCOUNT */
 static int mutt_sasl_cb_authname (void* context, int id, const char** result,
   unsigned* len)
 {
@@ -468,11 +393,19 @@ static int mutt_sasl_cb_authname (void* context, int id, const char** result,
 	      id == SASL_CB_AUTHNAME ? "authname" : "user",
 	      account->host, account->port));
 
-  if (mutt_account_getuser (account))
-    return SASL_FAIL;
-
-  *result = account->user;
-
+  if (id == SASL_CB_AUTHNAME)
+  {
+    if (mutt_account_getlogin (account))
+      return SASL_FAIL;
+    *result = account->login;
+  }
+  else
+  {
+    if (mutt_account_getuser (account))
+      return SASL_FAIL;
+    *result = account->user;
+  }
+  
   if (len)
     *len = strlen (*result);
 
@@ -489,7 +422,7 @@ static int mutt_sasl_cb_pass (sasl_conn_t* conn, void* context, int id,
     return SASL_BADPARAM;
 
   dprint (2, (debugfile,
-    "mutt_sasl_cb_pass: getting password for %s@%s:%u\n", account->user,
+    "mutt_sasl_cb_pass: getting password for %s@%s:%u\n", account->login,
     account->host, account->port));
 
   if (mutt_account_getpass (account))
@@ -499,7 +432,7 @@ static int mutt_sasl_cb_pass (sasl_conn_t* conn, void* context, int id,
 
   *psecret = (sasl_secret_t*) safe_malloc (sizeof (sasl_secret_t) + len);
   (*psecret)->len = len;
-  strcpy ((*psecret)->data, account->pass);	/* __STRCPY_CHECKED__ */
+  strcpy ((char*)(*psecret)->data, account->pass);	/* __STRCPY_CHECKED__ */
 
   return SASL_OK;
 }
@@ -539,9 +472,6 @@ static int mutt_sasl_conn_close (CONNECTION* conn)
 
   /* release sasl resources */
   sasl_dispose (&sasldata->saslconn);
-#ifndef USE_SASL2
-  FREE (&sasldata->buf);
-#endif
   FREE (&sasldata);
 
   /* call underlying close */
@@ -573,9 +503,6 @@ static int mutt_sasl_conn_read (CONNECTION* conn, char* buf, size_t len)
   
   conn->sockdata = sasldata->sockdata;
 
-#ifndef USE_SASL2
-  FREE (&sasldata->buf);
-#endif
   sasldata->bpos = 0;
   sasldata->blen = 0;
 
@@ -623,11 +550,7 @@ static int mutt_sasl_conn_write (CONNECTION* conn, const char* buf,
   SASL_DATA* sasldata;
   int rc;
 
-#ifdef USE_SASL2
   const char *pbuf;
-#else
-  char* pbuf;
-#endif
   unsigned int olen, plen;
 
   sasldata = (SASL_DATA*) conn->sockdata;
@@ -650,9 +573,6 @@ static int mutt_sasl_conn_write (CONNECTION* conn, const char* buf,
       }
 
       rc = (sasldata->msasl_write) (conn, pbuf, plen);
-#ifndef USE_SASL2
-      FREE (&pbuf);
-#endif
       if (rc != plen)
 	goto fail;
 
