@@ -26,7 +26,7 @@
 #include "globals.h"
 #include "mutt_socket.h"
 #include "mutt_tunnel.h"
-#if defined(USE_SSL) || defined(USE_GNUTLS) || defined(USE_NSS)
+#if defined(USE_SSL) || defined(USE_GNUTLS)
 # include "mutt_ssl.h"
 #endif
 
@@ -48,7 +48,7 @@ static CONNECTION *Connections = NULL;
 /* forward declarations */
 static int socket_preconnect (void);
 static int socket_connect (int fd, struct sockaddr* sa);
-static CONNECTION* socket_new_conn ();
+static CONNECTION* socket_new_conn (void);
 
 /* Wrappers */
 int mutt_socket_open (CONNECTION* conn) 
@@ -259,8 +259,6 @@ CONNECTION* mutt_conn_find (const CONNECTION* start, const ACCOUNT* account)
   {
 #ifdef USE_SSL
     ssl_socket_setup (conn);
-#elif USE_NSS
-    mutt_nss_socket_setup (conn);
 #elif USE_GNUTLS
     if (mutt_gnutls_socket_setup (conn) < 0)
     {
@@ -349,7 +347,7 @@ static int socket_connect (int fd, struct sockaddr* sa)
 }
 
 /* socket_new_conn: allocate and initialise a new connection. */
-static CONNECTION* socket_new_conn ()
+static CONNECTION* socket_new_conn (void)
 {
   CONNECTION* conn;
 
@@ -442,6 +440,7 @@ int raw_socket_open (CONNECTION* conn)
   if (rc)
   {
     mutt_error (_("Could not find the host \"%s\""), conn->account.host);
+    mutt_sleep (2);
     return -1;
   }
 
@@ -515,6 +514,7 @@ int raw_socket_open (CONNECTION* conn)
     {
       if ((rc = socket_connect (fd, (struct sockaddr*) &sin)) == 0)
       {
+        fcntl (fd, F_SETFD, FD_CLOEXEC);
 	conn->fd = fd;
 	break;
       }
