@@ -92,7 +92,7 @@ static int tls_socket_read (CONNECTION* conn, char* buf, size_t len)
 
   if (!data)
   {
-    mutt_error ("Error: no TLS socket open");
+    mutt_error (_("Error: no TLS socket open"));
     mutt_sleep (2);
     return -1;
   }
@@ -114,7 +114,7 @@ static int tls_socket_write (CONNECTION* conn, const char* buf, size_t len)
 
   if (!data)
   {
-    mutt_error ("Error: no TLS socket open");
+    mutt_error (_("Error: no TLS socket open"));
     mutt_sleep (2);
     return -1;
   }
@@ -342,8 +342,8 @@ static int tls_compare_certificates (const gnutls_datum *peercert)
       return 0;
     }
 
-    ptr = (unsigned char *)strstr(b64_data.data, CERT_SEP) + 1;
-    ptr = (unsigned char *)strstr(ptr, CERT_SEP);
+    ptr = (unsigned char *)strstr((char*)b64_data.data, CERT_SEP) + 1;
+    ptr = (unsigned char *)strstr((char*)ptr, CERT_SEP);
 
     b64_data.size = b64_data.size - (ptr - b64_data.data);
     b64_data.data = ptr;
@@ -474,10 +474,9 @@ static int tls_check_certificate (CONNECTION* conn)
   MUTTMENU *menu;
   int done, row, i, ret;
   FILE *fp;
-  gnutls_x509_dn dn;
   time_t t;
   const gnutls_datum *cert_list;
-  int cert_list_size = 0;
+  unsigned int cert_list_size = 0;
   gnutls_certificate_status certstat;
   char datestr[30];
   gnutls_x509_crt cert;
@@ -514,7 +513,7 @@ static int tls_check_certificate (CONNECTION* conn)
   /* We only support X.509 certificates (not OpenPGP) at the moment */
   if (gnutls_certificate_type_get(state) != GNUTLS_CRT_X509)
   {
-    mutt_error (_("Error certificate is not X.509"));
+    mutt_error (_("Certificate is not X.509"));
     mutt_sleep (2);
     return 0;
   }
@@ -775,6 +774,7 @@ static int tls_check_certificate (CONNECTION* conn)
   menu->help = helpstr;
 
   done = 0;
+  set_option (OPTUNBUFFEREDINPUT);
   while (!done)
   {
     switch (mutt_menuLoop (menu))
@@ -826,6 +826,7 @@ static int tls_check_certificate (CONNECTION* conn)
         break;
     }
   }
+  unset_option (OPTUNBUFFEREDINPUT);
   mutt_menuDestroy (&menu);
   gnutls_x509_crt_deinit(cert);
   return (done == 2);
