@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1999-2000 Thomas Roessler <roessler@does-not-exist.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,10 @@
  *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mx.h"
 #include "rfc2047.h"
@@ -26,10 +30,6 @@
 #include "keymap.h"
 #include "url.h"
 
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
-
 #ifdef USE_IMAP
 #include "imap.h"
 #endif
@@ -46,6 +46,8 @@
 #include "dotlock.h"
 #endif
 
+#include "mutt_crypt.h"
+
 #include <dirent.h>
 #include <fcntl.h>
 #include <sys/file.h>
@@ -117,7 +119,7 @@ retry_lock:
       
       snprintf(msg, sizeof(msg), _("Lock count exceeded, remove lock for %s?"),
 	       path);
-      if(retry && mutt_yesorno(msg, 1) == 1)
+      if(retry && mutt_yesorno(msg, M_YES) == M_YES)
       {
 	flags |= DL_FL_FORCE;
 	retry--;
@@ -158,13 +160,13 @@ int mx_lock_file (const char *path, int fd, int excl, int dot, int timeout)
 #ifdef USE_FCNTL
   struct flock lck;
   
-
   memset (&lck, 0, sizeof (struct flock));
   lck.l_type = excl ? F_WRLCK : F_RDLCK;
   lck.l_whence = SEEK_SET;
 
   count = 0;
   attempt = 0;
+  prev_sb.st_size = 0; /* silence a GCC warning */
   while (fcntl (fd, F_SETLK, &lck) == -1)
   {
     struct stat sb;
@@ -399,6 +401,10 @@ int mx_get_magic (const char *path)
     snprintf (tmp, sizeof (tmp), "%s/.mew-cache", path);
     if (access (tmp, F_OK) == 0)
       return (M_MH);
+    
+    snprintf (tmp, sizeof (tmp), "%s/.sylpheed_cache", path);
+    if (access (tmp, F_OK) == 0)
+      return (M_MH);
 
     /* 
      * ok, this isn't an mh folder, but mh mode can be used to read
@@ -464,8 +470,6 @@ int mx_set_magic (const char *s)
     DefaultMagic = M_MH;
   else if (ascii_strcasecmp (s, "maildir") == 0)
     DefaultMagic = M_MAILDIR;
-  else if (ascii_strcasecmp (s, "kendra") == 0)
-    DefaultMagic = M_KENDRA;
   else
     return (-1);
 
@@ -485,7 +489,7 @@ int mx_access (const char* path, int flags)
   return access (path, flags);
 }
 
-static int mx_open_mailbox_append (CONTEXT *ctx)
+static int mx_open_mailbox_append (CONTEXT *ctx, int flags)
 {
   struct stat sb;
 
@@ -581,7 +585,7 @@ static int mx_open_mailbox_append (CONTEXT *ctx)
   {
     case M_MBOX:
     case M_MMDF:
-      if ((ctx->fp = fopen (ctx->path, "a")) == NULL ||
+    if ((ctx->fp = safe_fopen (ctx->path, flags & M_NEWFOLDER ? "w" : "a")) == NULL ||
 	  mbox_lock_mailbox (ctx, 1, 1) != 0)
       {
 	if (!ctx->fp)
@@ -636,9 +640,9 @@ CONTEXT *mx_open_mailbox (const char *path, int flags, CONTEXT *pctx)
   if (flags & M_READONLY)
     ctx->readonly = 1;
 
-  if (flags & M_APPEND)
+  if (flags & (M_APPEND|M_NEWFOLDER))
   {
-    if (mx_open_mailbox_append (ctx) != 0)
+    if (mx_open_mailbox_append (ctx, flags) != 0)
     {
       mx_fastclose_mailbox (ctx);
       if (!pctx)
@@ -746,15 +750,17 @@ void mx_fastclose_mailbox (CONTEXT *ctx)
   if (ctx->magic == M_POP)
     pop_close_mailbox (ctx);
 #endif /* USE_POP */
+  if (ctx->subj_hash)
+    hash_destroy (&ctx->subj_hash, NULL);
   if (ctx->id_hash)
     hash_destroy (&ctx->id_hash, NULL);
   mutt_clear_threads (ctx);
   for (i = 0; i < ctx->msgcount; i++)
     mutt_free_header (&ctx->hdrs[i]);
-  safe_free ((void **) &ctx->hdrs);
-  safe_free ((void **) &ctx->v2r);
-  safe_free ((void **) &ctx->path);
-  safe_free ((void **) &ctx->pattern);
+  FREE (&ctx->hdrs);
+  FREE (&ctx->v2r);
+  FREE (&ctx->path);
+  FREE (&ctx->pattern);
   if (ctx->limit_pattern) 
     mutt_pattern_free (&ctx->limit_pattern);
   safe_fclose (&ctx->fp);
@@ -951,8 +957,7 @@ int mx_close_mailbox (CONTEXT *ctx, int *index_hint)
         {
 	  if (mutt_append_message (&f, ctx, ctx->hdrs[i], 0, CH_UPDATE_LEN) == 0)
 	  {
-	    ctx->hdrs[i]->deleted = 1;
-	    ctx->deleted++;
+	    mutt_set_flag (ctx, ctx->hdrs[i], M_DELETE, 1);
 	  }
 	  else
 	  {
@@ -1089,6 +1094,8 @@ void mx_update_tables(CONTEXT *ctx, int committing)
 		      ctx->hdrs[i]->content->offset -
 		      ctx->hdrs[i]->content->hdr_offset);
       /* remove message from the hash tables */
+      if (ctx->subj_hash && ctx->hdrs[i]->env->real_subj)
+	hash_delete (ctx->subj_hash, ctx->hdrs[i]->env->real_subj, ctx->hdrs[i], NULL);
       if (ctx->id_hash && ctx->hdrs[i]->env->message_id)
 	hash_delete (ctx->id_hash, ctx->hdrs[i]->env->message_id, ctx->hdrs[i], NULL);
       mutt_free_header (&ctx->hdrs[i]);
@@ -1158,6 +1165,8 @@ int mx_sync_mailbox (CONTEXT *ctx, int *index_hint)
         ctx->deleted = 0;
       }
     }
+    else if (ctx->last_tag && ctx->last_tag->deleted)
+      ctx->last_tag = NULL; /* reset last tagged msg now useless */
   }
 
   /* really only for IMAP - imap_sync_mailbox results in a call to
@@ -1231,7 +1240,10 @@ int imap_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
 
   mutt_mktemp(tmp);
   if ((msg->fp = safe_fopen (tmp, "w")) == NULL)
+  {
+    mutt_perror (tmp);
     return (-1);
+  }
   msg->path = safe_strdup(tmp);
   return 0;
 }
@@ -1247,7 +1259,6 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
   MESSAGE *msg;
   int (*func) (MESSAGE *, CONTEXT *, HEADER *);
   ADDRESS *p = NULL;
-  time_t t;
 
   switch (dest->magic)
   {
@@ -1281,7 +1292,11 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
     msg->flags.flagged = hdr->flagged;
     msg->flags.replied = hdr->replied;
     msg->flags.read    = hdr->read;
+    msg->received = hdr->received;
   }
+
+  if(msg->received == 0)
+    time(&msg->received);
   
   if (func (msg, dest, hdr) == 0)
   {
@@ -1299,19 +1314,13 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
 	  p = hdr->env->sender;
 	else
 	  p = hdr->env->from;
-
-	if (!hdr->received)
-	  hdr->received = time (NULL);
-	t = hdr->received;
       }
-      else
-	t = time (NULL);
 
-      fprintf (msg->fp, "From %s %s", p ? p->mailbox : NONULL(Username), ctime (&t));
+      fprintf (msg->fp, "From %s %s", p ? p->mailbox : NONULL(Username), ctime (&msg->received));
     }
   }
   else
-    safe_free ((void **) &msg);
+    FREE (&msg);
 
   return msg;
 }
@@ -1352,12 +1361,13 @@ int mx_check_mailbox (CONTEXT *ctx, int *index_hint, int lock)
 
 
       case M_MH:
-      case M_MAILDIR:
 	return (mh_check_mailbox (ctx, index_hint));
+      case M_MAILDIR:
+	return (maildir_check_mailbox (ctx, index_hint));
 
 #ifdef USE_IMAP
       case M_IMAP:
-	return (imap_check_mailbox (ctx, index_hint));
+	return (imap_check_mailbox (ctx, index_hint, 0));
 #endif /* USE_IMAP */
 
 #ifdef USE_POP
@@ -1461,13 +1471,6 @@ int mx_commit_message (MESSAGE *msg, CONTEXT *ctx)
       break;
     }
 
-    case M_KENDRA:
-    {
-      if (fputs (KENDRA_SEP, msg->fp) == EOF)
-	r = -1;
-      break;
-    }
-
 #ifdef USE_IMAP
     case M_IMAP:
     {
@@ -1490,7 +1493,7 @@ int mx_commit_message (MESSAGE *msg, CONTEXT *ctx)
     }
   }
   
-  if (r == 0 && (ctx->magic == M_MBOX || ctx->magic == M_MMDF || ctx->magic == M_KENDRA)
+  if (r == 0 && (ctx->magic == M_MBOX || ctx->magic == M_MMDF)
       && (fflush (msg->fp) == EOF || fsync (fileno (msg->fp)) == -1))
   {
     mutt_perror _("Can't write message");
@@ -1534,16 +1537,24 @@ int mx_close_message (MESSAGE **msg)
 void mx_alloc_memory (CONTEXT *ctx)
 {
   int i;
-
+  size_t s = MAX (sizeof (HEADER *), sizeof (int));
+  
+  if ((ctx->hdrmax + 25) * s < ctx->hdrmax * s)
+  {
+    mutt_error _("Integer overflow -- can't allocate memory.");
+    sleep (1);
+    mutt_exit (1);
+  }
+  
   if (ctx->hdrs)
   {
-    safe_realloc ((void **) &ctx->hdrs, sizeof (HEADER *) * (ctx->hdrmax += 25));
-    safe_realloc ((void **) &ctx->v2r, sizeof (int) * ctx->hdrmax);
+    safe_realloc (&ctx->hdrs, sizeof (HEADER *) * (ctx->hdrmax += 25));
+    safe_realloc (&ctx->v2r, sizeof (int) * ctx->hdrmax);
   }
   else
   {
-    ctx->hdrs = safe_malloc (sizeof (HEADER *) * (ctx->hdrmax += 25));
-    ctx->v2r = safe_malloc (sizeof (int) * ctx->hdrmax);
+    ctx->hdrs = safe_calloc ((ctx->hdrmax += 25), sizeof (HEADER *));
+    ctx->v2r = safe_calloc (ctx->hdrmax, sizeof (int));
   }
   for (i = ctx->msgcount ; i < ctx->hdrmax ; i++)
   {
@@ -1564,12 +1575,11 @@ void mx_update_context (CONTEXT *ctx, int new_messages)
   {
     h = ctx->hdrs[msgno];
 
-
-
-#ifdef HAVE_PGP
-    /* NOTE: this _must_ be done before the check for mailcap! */
-    h->pgp = pgp_query (h->content);
-#endif /* HAVE_PGP */
+    if (WithCrypto)
+    {
+      /* NOTE: this _must_ be done before the check for mailcap! */
+      h->security = crypt_query (h->content);
+    }
 
     if (!ctx->pattern)
     {
@@ -1589,7 +1599,7 @@ void mx_update_context (CONTEXT *ctx, int new_messages)
 
       h2 = hash_find (ctx->id_hash, h->env->supersedes);
 
-      /* safe_free (&h->env->supersedes); should I ? */
+      /* FREE (&h->env->supersedes); should I ? */
       if (h2)
       {
 	h2->superseded = 1;
@@ -1601,6 +1611,8 @@ void mx_update_context (CONTEXT *ctx, int new_messages)
     /* add this message to the hash tables */
     if (ctx->id_hash && h->env->message_id)
       hash_insert (ctx->id_hash, h->env->message_id, h, 0);
+    if (ctx->subj_hash && h->env->real_subj)
+      hash_insert (ctx->subj_hash, h->env->real_subj, h, 1);
 
     if (option (OPTSCORE)) 
       mutt_score_message (ctx, h, 0);
@@ -1619,3 +1631,27 @@ void mx_update_context (CONTEXT *ctx, int new_messages)
     }
   }
 }
+
+/*
+ * Return:
+ * 1 if the specified mailbox contains 0 messages.
+ * 0 if the mailbox contains messages
+ * -1 on error
+ */
+int mx_check_empty (const char *path)
+{
+  switch (mx_get_magic (path))
+  {
+    case M_MBOX:
+    case M_MMDF:
+      return mbox_check_empty (path);
+    case M_MH:
+      return mh_check_empty (path);
+    case M_MAILDIR:
+      return maildir_check_empty (path);
+    default:
+      errno = EINVAL;
+      return -1;
+  }
+  /* not reached */
+}
