@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,22 +13,21 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
  */ 
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "mutt.h"
 #include "mutt_regex.h"
 #include "mailbox.h"
 #include "mime.h"
 #include "rfc2047.h"
-
-
-
-#ifdef _PGPPATH
-#include "pgp.h"
-#endif /* _PGPPATH */
-
-
+#include "rfc2231.h"
+#include "mutt_crypt.h"
+#include "url.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -39,7 +38,7 @@
  * lines.  ``line'' must point to a dynamically allocated string; it is
  * increased if more space is required to fit the whole line.
  */
-static char *read_rfc822_line (FILE *f, char *line, size_t *linelen)
+char *mutt_read_rfc822_line (FILE *f, char *line, size_t *linelen)
 {
   char *buf = line;
   char ch;
@@ -83,30 +82,72 @@ static char *read_rfc822_line (FILE *f, char *line, size_t *linelen)
     {
       /* grow the buffer */
       *linelen += STRING;
-      safe_realloc ((void **) &line, *linelen);
+      safe_realloc (&line, *linelen);
       buf = line + offset;
     }
   }
   /* not reached */
 }
 
-static LIST *mutt_parse_references (char *s)
+static LIST *mutt_parse_references (char *s, int in_reply_to)
 {
   LIST *t, *lst = NULL;
+  int m, n = 0;
+  char *o = NULL, *new, *at;
 
-  while ((s = strtok (s, " \t")) != NULL)
+  while ((s = strtok (s, " \t;")) != NULL)
   {
     /*
      * some mail clients add other garbage besides message-ids, so do a quick
      * check to make sure this looks like a valid message-id
+     * some idiotic clients also break their message-ids between lines, deal
+     * with that too (give up if it's more than two lines, though)
      */
+    t = NULL;
+    new = NULL;
+
     if (*s == '<')
     {
-      t = (LIST *)safe_malloc (sizeof (LIST));
-      t->data = safe_strdup (s);
-      t->next = lst;
-      lst = t;
+      n = strlen (s);
+      if (s[n-1] != '>')
+      {
+	o = s;
+	s = NULL;
+	continue;
+      }
+
+      new = safe_strdup (s);
     }
+    else if (o)
+    {
+      m = strlen (s);
+      if (s[m - 1] == '>')
+      {
+	new = safe_malloc (sizeof (char) * (n + m + 1));
+	strcpy (new, o);	/* __STRCPY_CHECKED__ */
+	strcpy (new + n, s);	/* __STRCPY_CHECKED__ */
+      }
+    }
+    if (new)
+    {
+      /* make sure that this really does look like a message-id.
+       * it should have exactly one @, and if we're looking at
+       * an in-reply-to header, make sure that the part before
+       * the @ has more than eight characters or it's probably
+       * an email address
+       */
+      if (!(at = strchr (new, '@')) || strchr (at + 1, '@')
+	  || (in_reply_to && at - new <= 8))
+	FREE (&new);
+      else
+      {
+	t = (LIST *) safe_malloc (sizeof (LIST));
+	t->data = new;
+	t->next = lst;
+	lst = t;
+      }
+    }
+    o = NULL;
     s = NULL;
   }
 
@@ -115,18 +156,22 @@ static LIST *mutt_parse_references (char *s)
 
 int mutt_check_encoding (const char *c)
 {
-  if (mutt_strncasecmp ("7bit", c, sizeof ("7bit")-1) == 0)
+  if (ascii_strncasecmp ("7bit", c, sizeof ("7bit")-1) == 0)
     return (ENC7BIT);
-  else if (mutt_strncasecmp ("8bit", c, sizeof ("8bit")-1) == 0)
+  else if (ascii_strncasecmp ("8bit", c, sizeof ("8bit")-1) == 0)
     return (ENC8BIT);
-  else if (mutt_strncasecmp ("binary", c, sizeof ("binary")-1) == 0)
+  else if (ascii_strncasecmp ("binary", c, sizeof ("binary")-1) == 0)
     return (ENCBINARY);
-  else if (mutt_strncasecmp ("quoted-printable", c, sizeof ("quoted-printable")-1) == 0)
+  else if (ascii_strncasecmp ("quoted-printable", c, sizeof ("quoted-printable")-1) == 0)
     return (ENCQUOTEDPRINTABLE);
-  else if (mutt_strncasecmp ("base64", c, sizeof("base64")-1) == 0)
+  else if (ascii_strncasecmp ("base64", c, sizeof("base64")-1) == 0)
     return (ENCBASE64);
-  else if (mutt_strncasecmp ("x-uuencode", c, sizeof("x-uuencode")-1) == 0)
+  else if (ascii_strncasecmp ("x-uuencode", c, sizeof("x-uuencode")-1) == 0)
     return (ENCUUENCODED);
+#ifdef SUN_ATTACHMENT
+  else if (ascii_strncasecmp ("uuencode", c, sizeof("uuencode")-1) == 0)
+    return (ENCUUENCODED);
+#endif
   else
     return (ENCOTHER);
 }
@@ -138,12 +183,14 @@ static PARAMETER *parse_parameters (const char *s)
   const char *p;
   size_t i;
 
+  dprint (2, (debugfile, "parse_parameters: `%s'\n", s));
+  
   while (*s)
   {
     if ((p = strpbrk (s, "=;")) == NULL)
     {
       dprint(1, (debugfile, "parse_parameters: malformed parameter: %s\n", s));
-      return (head); /* just bail out now */
+      goto bail;
     }
 
     /* if we hit a ; now the parameter has no value, just skip it */
@@ -192,6 +239,10 @@ static PARAMETER *parse_parameters (const char *s)
 
       new->value = safe_strdup (buffer);
 
+      dprint (2, (debugfile, "parse_parameter: `%s' = `%s'\n",
+		  new->attribute ? new->attribute : "",
+		  new->value ? new->value : ""));
+      
       /* Add this parameter to the list */
       if (head)
       {
@@ -221,25 +272,34 @@ static PARAMETER *parse_parameters (const char *s)
     while (*s == ';'); /* skip empty parameters */
   }    
 
+  bail:
+
+  rfc2231_decode_parameters (&head);
   return (head);
 }
 
 int mutt_check_mime_type (const char *s)
 {
-  if (mutt_strcasecmp ("text", s) == 0)
+  if (ascii_strcasecmp ("text", s) == 0)
     return TYPETEXT;
-  else if (mutt_strcasecmp ("multipart", s) == 0)
+  else if (ascii_strcasecmp ("multipart", s) == 0)
     return TYPEMULTIPART;
-  else if (mutt_strcasecmp ("application", s) == 0)
+#ifdef SUN_ATTACHMENT 
+  else if (ascii_strcasecmp ("x-sun-attachment", s) == 0)
+    return TYPEMULTIPART;
+#endif
+  else if (ascii_strcasecmp ("application", s) == 0)
     return TYPEAPPLICATION;
-  else if (mutt_strcasecmp ("message", s) == 0)
+  else if (ascii_strcasecmp ("message", s) == 0)
     return TYPEMESSAGE;
-  else if (mutt_strcasecmp ("image", s) == 0)
+  else if (ascii_strcasecmp ("image", s) == 0)
     return TYPEIMAGE;
-  else if (mutt_strcasecmp ("audio", s) == 0)
+  else if (ascii_strcasecmp ("audio", s) == 0)
     return TYPEAUDIO;
-  else if (mutt_strcasecmp ("video", s) == 0)
+  else if (ascii_strcasecmp ("video", s) == 0)
     return TYPEVIDEO;
+  else if (ascii_strcasecmp ("model", s) == 0)
+    return TYPEMODEL;
   else
     return TYPEOTHER;
 }
@@ -249,7 +309,7 @@ void mutt_parse_content_type (char *s, BODY *ct)
   char *pc;
   char *subtype;
 
-  safe_free((void **)&ct->subtype);
+  FREE (&ct->subtype);
   mutt_free_parameter(&ct->parameter);
 
   /* First extract any existing parameters */
@@ -260,9 +320,18 @@ void mutt_parse_content_type (char *s, BODY *ct)
       pc++;
     ct->parameter = parse_parameters(pc);
 
-    /* Some pre-RFC1521 gateways still use the "name=filename" convention */
-    if ((pc = mutt_get_parameter("name", ct->parameter)) != 0)
+    /* Some pre-RFC1521 gateways still use the "name=filename" convention,
+     * but if a filename has already been set in the content-disposition,
+     * let that take precedence, and don't set it here */
+    if ((pc = mutt_get_parameter( "name", ct->parameter)) != 0 && !ct->filename)
       ct->filename = safe_strdup(pc);
+    
+#ifdef SUN_ATTACHMENT
+    /* this is deep and utter perversion */
+    if ((pc = mutt_get_parameter ("conversions", ct->parameter)) != 0)
+      ct->encoding = mutt_check_encoding (pc);
+#endif
+    
   }
   
   /* Now get the subtype */
@@ -278,6 +347,11 @@ void mutt_parse_content_type (char *s, BODY *ct)
   /* Finally, get the major type */
   ct->type = mutt_check_mime_type (s);
 
+#ifdef SUN_ATTACHMENT
+  if (ascii_strcasecmp ("x-sun-attachment", s) == 0)
+      ct->subtype = safe_strdup ("x-sun-attachment");
+#endif
+
   if (ct->type == TYPEOTHER)
   {
     ct->xtype = safe_strdup (s);
@@ -306,15 +380,22 @@ void mutt_parse_content_type (char *s, BODY *ct)
       ct->subtype = safe_strdup ("x-unknown");
   }
 
+  /* Default character set for text types. */
+  if (ct->type == TYPETEXT)
+  {
+    if (!(pc = mutt_get_parameter ("charset", ct->parameter)))
+      mutt_set_parameter ("charset", "us-ascii", &ct->parameter);
+  }
+
 }
 
 static void parse_content_disposition (char *s, BODY *ct)
 {
   PARAMETER *parms;
 
-  if (!mutt_strncasecmp ("inline", s, 6))
+  if (!ascii_strncasecmp ("inline", s, 6))
     ct->disposition = DISPINLINE;
-  else if (!mutt_strncasecmp ("form-data", s, 9))
+  else if (!ascii_strncasecmp ("form-data", s, 9))
     ct->disposition = DISPFORMDATA;
   else
     ct->disposition = DISPATTACH;
@@ -325,14 +406,7 @@ static void parse_content_disposition (char *s, BODY *ct)
     s++;
     SKIPWS (s);
     if ((s = mutt_get_parameter ("filename", (parms = parse_parameters (s)))) != 0)
-    {
-      /* free() here because the content-type parsing routine might
-       * have allocated space if a "name=filename" parameter was
-       * specified.
-       */
-      safe_free ((void **) &ct->filename);
-      ct->filename = safe_strdup (s); 
-    }
+      mutt_str_replace (&ct->filename, s);
     if ((s = mutt_get_parameter ("name", parms)) != 0)
       ct->form_name = safe_strdup (s);
     mutt_free_parameter (&parms);
@@ -353,12 +427,13 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
   char *line = safe_malloc (LONG_STRING);
   size_t linelen = LONG_STRING;
   
-  p->hdr_offset = ftell(fp);
+  p->hdr_offset  = ftell(fp);
 
-  p->encoding = ENC7BIT; /* default from RFC1521 */
-  p->type = digest ? TYPEMESSAGE : TYPETEXT;
-
-  while (*(line = read_rfc822_line (fp, line, &linelen)) != 0)
+  p->encoding    = ENC7BIT; /* default from RFC1521 */
+  p->type        = digest ? TYPEMESSAGE : TYPETEXT;
+  p->disposition = DISPINLINE;
+  
+  while (*(line = mutt_read_rfc822_line (fp, line, &linelen)) != 0)
   {
     /* Find the value of the current header */
     if ((c = strchr (line, ':')))
@@ -378,21 +453,36 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
       break;
     }
 
-    if (!mutt_strncasecmp ("content-", line, 8))
+    if (!ascii_strncasecmp ("content-", line, 8))
     {
-      if (!mutt_strcasecmp ("type", line + 8))
+      if (!ascii_strcasecmp ("type", line + 8))
 	mutt_parse_content_type (c, p);
-      else if (!mutt_strcasecmp ("transfer-encoding", line + 8))
+      else if (!ascii_strcasecmp ("transfer-encoding", line + 8))
 	p->encoding = mutt_check_encoding (c);
-      else if (!mutt_strcasecmp ("disposition", line + 8))
+      else if (!ascii_strcasecmp ("disposition", line + 8))
 	parse_content_disposition (c, p);
-      else if (!mutt_strcasecmp ("description", line + 8))
+      else if (!ascii_strcasecmp ("description", line + 8))
       {
-	safe_free ((void **) &p->description);
-	p->description = safe_strdup (c);
-	rfc2047_decode (p->description, p->description, mutt_strlen (p->description) + 1);
+	mutt_str_replace (&p->description, c);
+	rfc2047_decode (&p->description);
+      }
+    } 
+#ifdef SUN_ATTACHMENT
+    else if (!ascii_strncasecmp ("x-sun-", line, 6))
+    {
+      if (!ascii_strcasecmp ("data-type", line + 6))
+        mutt_parse_content_type (c, p);
+      else if (!ascii_strcasecmp ("encoding-info", line + 6))
+        p->encoding = mutt_check_encoding (c);
+      else if (!ascii_strcasecmp ("content-lines", line + 6))
+        mutt_set_parameter ("content-lines", c, &(p->parameter));
+      else if (!ascii_strcasecmp ("data-description", line + 6))
+      {
+	mutt_str_replace (&p->description, c);
+        rfc2047_decode (&p->description);
       }
     }
+#endif
   }
   p->offset = ftell (fp); /* Mark the start of the real data */
   if (p->type == TYPETEXT && !p->subtype)
@@ -407,13 +497,22 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
 
 void mutt_parse_part (FILE *fp, BODY *b)
 {
+  char *bound = 0;
+
   switch (b->type)
   {
     case TYPEMULTIPART:
+#ifdef SUN_ATTACHMENT
+      if ( !ascii_strcasecmp (b->subtype, "x-sun-attachment") )
+          bound = "--------";
+      else
+#endif
+          bound = mutt_get_parameter ("boundary", b->parameter);
+
       fseek (fp, b->offset, SEEK_SET);
-      b->parts =  mutt_parse_multipart (fp, mutt_get_parameter ("boundary", b->parameter), 
+      b->parts =  mutt_parse_multipart (fp, bound, 
 					b->offset + b->length,
-					mutt_strcasecmp ("digest", b->subtype) == 0);
+					ascii_strcasecmp ("digest", b->subtype) == 0);
       break;
 
     case TYPEMESSAGE:
@@ -422,7 +521,7 @@ void mutt_parse_part (FILE *fp, BODY *b)
 	fseek (fp, b->offset, SEEK_SET);
 	if (mutt_is_message_type(b->type, b->subtype))
 	  b->parts = mutt_parse_messageRFC822 (fp, b);
-	else if (mutt_strcasecmp (b->subtype, "external-body") == 0)
+	else if (ascii_strcasecmp (b->subtype, "external-body") == 0)
 	  b->parts = mutt_read_mime_header (fp, 0);
 	else
 	  return;
@@ -437,8 +536,7 @@ void mutt_parse_part (FILE *fp, BODY *b)
   if (!b->parts)
   {
     b->type = TYPETEXT;
-    safe_free ((void **) &b->subtype);
-    b->subtype = safe_strdup ("plain");
+    mutt_str_replace (&b->subtype, "plain");
   }
 }
 
@@ -459,7 +557,7 @@ BODY *mutt_parse_messageRFC822 (FILE *fp, BODY *parent)
 
   parent->hdr = mutt_new_header ();
   parent->hdr->offset = ftell (fp);
-  parent->hdr->env = mutt_read_rfc822_header (fp, parent->hdr, 0);
+  parent->hdr->env = mutt_read_rfc822_header (fp, parent->hdr, 0, 0);
   msg = parent->hdr->content;
 
   /* ignore the length given in the content-length since it could be wrong
@@ -490,6 +588,9 @@ BODY *mutt_parse_messageRFC822 (FILE *fp, BODY *parent)
 
 BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int digest)
 {
+#ifdef SUN_ATTACHMENT
+  int lines;
+#endif
   int blen, len, crlf = 0;
   char buffer[LONG_STRING];
   BODY *head = 0, *last = 0, *new = 0;
@@ -507,11 +608,7 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int di
   {
     len = mutt_strlen (buffer);
 
-    /* take note of the line ending.  I'm assuming that either all endings
-     * will use <CR><LF> or none will.
-     */
-    if (len > 1 && buffer[len - 2] == '\r')
-      crlf = 1;
+    crlf =  (len > 1 && buffer[len - 2] == '\r') ? 1 : 0;
 
     if (buffer[0] == '-' && buffer[1] == '-' &&
 	mutt_strncmp (buffer + 2, boundary, blen) == 0)
@@ -539,6 +636,15 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int di
       else if (buffer[2 + blen] == 0)
       {
 	new = mutt_read_mime_header (fp, digest);
+
+#ifdef SUN_ATTACHMENT
+        if (mutt_get_parameter ("content-lines", new->parameter)) {
+	  for (lines = atoi(mutt_get_parameter ("content-lines", new->parameter));
+	       lines; lines-- )
+	     if (ftell (fp) >= end_off || fgets (buffer, LONG_STRING, fp) == NULL)
+	       break;
+	}
+#endif
 	
 	/*
 	 * Consistency checking - catch
@@ -707,7 +813,9 @@ time_t mutt_parse_date (const char *s, HEADER *h)
 
       case 2: /* year */
 	tm.tm_year = atoi (t);
-	if (tm.tm_year >= 1900)
+        if (tm.tm_year < 50)
+	  tm.tm_year += 100;
+        else if (tm.tm_year >= 1900)
 	  tm.tm_year -= 1900;
 	break;
 
@@ -751,7 +859,7 @@ time_t mutt_parse_date (const char *s, HEADER *h)
 
 	  tz = bsearch (ptz, TimeZones, sizeof TimeZones/sizeof (struct tz_t),
 			sizeof (struct tz_t),
-			(int (*)(const void *, const void *)) strcasecmp
+			(int (*)(const void *, const void *)) ascii_strcasecmp
 			/* This is safe to do: A pointer to a struct equals
 			 * a pointer to its first element*/);
 
@@ -763,11 +871,11 @@ time_t mutt_parse_date (const char *s, HEADER *h)
 	  }
 
 	  /* ad hoc support for the European MET (now officially CET) TZ */
-	  if (mutt_strcasecmp (t, "MET") == 0)
+	  if (ascii_strcasecmp (t, "MET") == 0)
 	  {
 	    if ((t = strtok (NULL, " \t")) != NULL)
 	    {
-	      if (!mutt_strcasecmp (t, "DST"))
+	      if (!ascii_strcasecmp (t, "DST"))
 		zhours++;
 	    }
 	  }
@@ -827,17 +935,340 @@ void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
   {
     mutt_parse_part (msg->fp, cur->content);
 
-
-#ifdef _PGPPATH
-    cur->pgp = pgp_query (cur->content);
-#endif /* _PGPPATH */
-
-
+    if (WithCrypto)
+      cur->security = crypt_query (cur->content);
 
     mx_close_message (&msg);
   }
 }
 
+int mutt_parse_rfc822_line (ENVELOPE *e, HEADER *hdr, char *line, char *p, short user_hdrs, short weed,
+			    short do_2047, LIST **lastp)
+{
+  int matched = 0;
+  LIST *last = NULL;
+  
+  if (lastp)
+    last = *lastp;
+  
+  switch (ascii_tolower (line[0]))
+  {
+    case 'a':
+    if (ascii_strcasecmp (line+1, "pparently-to") == 0)
+    {
+      e->to = rfc822_parse_adrlist (e->to, p);
+      matched = 1;
+    }
+    else if (ascii_strcasecmp (line+1, "pparently-from") == 0)
+    {
+      e->from = rfc822_parse_adrlist (e->from, p);
+      matched = 1;
+    }
+    break;
+    
+    case 'b':
+    if (ascii_strcasecmp (line+1, "cc") == 0)
+    {
+      e->bcc = rfc822_parse_adrlist (e->bcc, p);
+      matched = 1;
+    }
+    break;
+    
+    case 'c':
+    if (ascii_strcasecmp (line+1, "c") == 0)
+    {
+      e->cc = rfc822_parse_adrlist (e->cc, p);
+      matched = 1;
+    }
+    else if (ascii_strncasecmp (line + 1, "ontent-", 7) == 0)
+    {
+      if (ascii_strcasecmp (line+8, "type") == 0)
+      {
+	if (hdr)
+	  mutt_parse_content_type (p, hdr->content);
+	matched = 1;
+      }
+      else if (ascii_strcasecmp (line+8, "transfer-encoding") == 0)
+      {
+	if (hdr)
+	  hdr->content->encoding = mutt_check_encoding (p);
+	matched = 1;
+      }
+      else if (ascii_strcasecmp (line+8, "length") == 0)
+      {
+	if (hdr)
+	{
+	  if ((hdr->content->length = atoi (p)) < 0)
+	    hdr->content->length = -1;
+	}
+	matched = 1;
+      }
+      else if (ascii_strcasecmp (line+8, "description") == 0)
+      {
+	if (hdr)
+	{
+	  mutt_str_replace (&hdr->content->description, p);
+	  rfc2047_decode (&hdr->content->description);
+	}
+	matched = 1;
+      }
+      else if (ascii_strcasecmp (line+8, "disposition") == 0)
+      {
+	if (hdr)
+	  parse_content_disposition (p, hdr->content);
+	matched = 1;
+      }
+    }
+    break;
+    
+    case 'd':
+    if (!ascii_strcasecmp ("ate", line + 1))
+    {
+      mutt_str_replace (&e->date, p);
+      if (hdr)
+	hdr->date_sent = mutt_parse_date (p, hdr);
+      matched = 1;
+    }
+    break;
+    
+    case 'e':
+    if (!ascii_strcasecmp ("xpires", line + 1) &&
+	hdr && mutt_parse_date (p, NULL) < time (NULL))
+      hdr->expired = 1;
+    break;
+    
+    case 'f':
+    if (!ascii_strcasecmp ("rom", line + 1))
+    {
+      e->from = rfc822_parse_adrlist (e->from, p);
+      matched = 1;
+    }
+    break;
+    
+    case 'i':
+    if (!ascii_strcasecmp (line+1, "n-reply-to"))
+    {
+      mutt_free_list (&e->in_reply_to);
+      e->in_reply_to = mutt_parse_references (p, 1);
+      matched = 1;
+    }
+    break;
+    
+    case 'l':
+    if (!ascii_strcasecmp (line + 1, "ines"))
+    {
+      if (hdr)
+      {
+	hdr->lines = atoi (p);
+
+	/* 
+	 * HACK - mutt has, for a very short time, produced negative
+	 * Lines header values.  Ignore them. 
+	 */
+	if (hdr->lines < 0)
+	  hdr->lines = 0;
+      }
+
+      matched = 1;
+    }
+    else if (!ascii_strcasecmp (line + 1, "ist-Post"))
+    {
+      /* RFC 2369.  FIXME: We should ignore whitespace, but don't. */
+      if (strncmp (p, "NO", 2))
+      {
+	char *beg, *end;
+	for (beg = strchr (p, '<'); beg; beg = strchr (end, ','))
+	{
+	  ++beg;
+	  if (!(end = strchr (beg, '>')))
+	    break;
+	  
+	  /* Take the first mailto URL */
+	  if (url_check_scheme (beg) == U_MAILTO)
+	  {
+	    FREE (&e->list_post);
+	    e->list_post = mutt_substrdup (beg, end);
+	    break;
+	  }
+	}
+      }
+      matched = 1;
+    }
+    break;
+    
+    case 'm':
+    if (!ascii_strcasecmp (line + 1, "ime-version"))
+    {
+      if (hdr)
+	hdr->mime = 1;
+      matched = 1;
+    }
+    else if (!ascii_strcasecmp (line + 1, "essage-id"))
+    {
+      /* We add a new "Message-Id:" when building a message */
+      FREE (&e->message_id);
+      e->message_id = extract_message_id (p);
+      matched = 1;
+    }
+    else if (!ascii_strncasecmp (line + 1, "ail-", 4))
+    {
+      if (!ascii_strcasecmp (line + 5, "reply-to"))
+      {
+	/* override the Reply-To: field */
+	rfc822_free_address (&e->reply_to);
+	e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
+	matched = 1;
+      }
+      else if (!ascii_strcasecmp (line + 5, "followup-to"))
+      {
+	e->mail_followup_to = rfc822_parse_adrlist (e->mail_followup_to, p);
+	matched = 1;
+      }
+    }
+    break;
+    
+    case 'r':
+    if (!ascii_strcasecmp (line + 1, "eferences"))
+    {
+      mutt_free_list (&e->references);
+      e->references = mutt_parse_references (p, 0);
+      matched = 1;
+    }
+    else if (!ascii_strcasecmp (line + 1, "eply-to"))
+    {
+      e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
+      matched = 1;
+    }
+    else if (!ascii_strcasecmp (line + 1, "eturn-path"))
+    {
+      e->return_path = rfc822_parse_adrlist (e->return_path, p);
+      matched = 1;
+    }
+    else if (!ascii_strcasecmp (line + 1, "eceived"))
+    {
+      if (hdr && !hdr->received)
+      {
+	char *d = strchr (p, ';');
+	
+	if (d)
+	  hdr->received = mutt_parse_date (d + 1, NULL);
+      }
+    }
+    break;
+    
+    case 's':
+    if (!ascii_strcasecmp (line + 1, "ubject"))
+    {
+      if (!e->subject)
+	e->subject = safe_strdup (p);
+      matched = 1;
+    }
+    else if (!ascii_strcasecmp (line + 1, "ender"))
+    {
+      e->sender = rfc822_parse_adrlist (e->sender, p);
+      matched = 1;
+    }
+    else if (!ascii_strcasecmp (line + 1, "tatus"))
+    {
+      if (hdr)
+      {
+	while (*p)
+	{
+	  switch(*p)
+	  {
+	    case 'r':
+	    hdr->replied = 1;
+	    break;
+	    case 'O':
+	      hdr->old = 1;
+	    break;
+	    case 'R':
+	    hdr->read = 1;
+	    break;
+	  }
+	  p++;
+	}
+      }
+      matched = 1;
+    }
+    else if ((!ascii_strcasecmp ("upersedes", line + 1) ||
+	      !ascii_strcasecmp ("upercedes", line + 1)) && hdr)
+      e->supersedes = safe_strdup (p);
+    break;
+    
+    case 't':
+    if (ascii_strcasecmp (line+1, "o") == 0)
+    {
+      e->to = rfc822_parse_adrlist (e->to, p);
+      matched = 1;
+    }
+    break;
+    
+    case 'x':
+    if (ascii_strcasecmp (line+1, "-status") == 0)
+    {
+      if (hdr)
+      {
+	while (*p)
+	{
+	  switch (*p)
+	  {
+	    case 'A':
+	    hdr->replied = 1;
+	    break;
+	    case 'D':
+	    hdr->deleted = 1;
+	    break;
+	    case 'F':
+	    hdr->flagged = 1;
+	    break;
+	    default:
+	    break;
+	  }
+	  p++;
+	}
+      }
+      matched = 1;
+    }
+    else if (ascii_strcasecmp (line+1, "-label") == 0)
+    {
+      e->x_label = safe_strdup(p);
+      matched = 1;
+    }
+    
+    default:
+    break;
+  }
+  
+  /* Keep track of the user-defined headers */
+  if (!matched && user_hdrs)
+  {
+    /* restore the original line */
+    line[strlen (line)] = ':';
+    
+    if (weed && option (OPTWEED) && mutt_matches_ignore (line, Ignore)
+	&& !mutt_matches_ignore (line, UnIgnore))
+      goto done;
+
+    if (last)
+    {
+      last->next = mutt_new_list ();
+      last = last->next;
+    }
+    else
+      last = e->userhdrs = mutt_new_list ();
+    last->data = safe_strdup (line);
+    if (do_2047)
+      rfc2047_decode (&last->data);
+  }
+
+  done:
+  
+  *lastp = last;
+  return matched;
+}
+  
+  
 /* mutt_read_rfc822_header() -- parses a RFC822 header
  *
  * Args:
@@ -846,36 +1277,47 @@ void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
  *
  * hdr		header structure of current message (optional).
  * 
- * user_hdrs	If set, store user headers.  Used for edit-message and 
+ * user_hdrs	If set, store user headers.  Used for recall-message and
  * 		postpone modes.
  * 
+ * weed		If this parameter is set and the user has activated the
+ * 		$weed option, honor the header weed list for user headers.
+ * 	        Used for recall-message.
+ * 
+ * Returns:     newly allocated envelope structure.  You should free it by
+ *              mutt_free_envelope() when envelope stay unneeded.
  */
-ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs)
+ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
+				   short weed)
 {
   ENVELOPE *e = mutt_new_envelope();
   LIST *last = NULL;
   char *line = safe_malloc (LONG_STRING);
   char *p;
-  char in_reply_to[STRING];
   long loc;
   int matched;
   size_t linelen = LONG_STRING;
-
-  in_reply_to[0] = 0;
+  char buf[LONG_STRING+1];
 
   if (hdr)
   {
-    hdr->content = mutt_new_body ();
+    if (hdr->content == NULL)
+    {
+      hdr->content = mutt_new_body ();
 
-    /* set the defaults from RFC1521 */
-    hdr->content->type = TYPETEXT;
-    hdr->content->subtype = safe_strdup ("plain");
-    hdr->content->encoding = ENC7BIT;
-    hdr->content->length = -1;
+      /* set the defaults from RFC1521 */
+      hdr->content->type        = TYPETEXT;
+      hdr->content->subtype     = safe_strdup ("plain");
+      hdr->content->encoding    = ENC7BIT;
+      hdr->content->length      = -1;
+
+      /* RFC 2183 says this is arbitrary */
+      hdr->content->disposition = DISPINLINE;
+    }
   }
 
-  loc = ftell (f);
-  while (*(line = read_rfc822_line (f, line, &linelen)) != 0)
+  while ((loc = ftell (f)),
+	  *(line = mutt_read_rfc822_line (f, line, &linelen)) != 0)
   {
     matched = 0;
 
@@ -886,16 +1328,12 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs)
 
       /* some bogus MTAs will quote the original "From " line */
       if (mutt_strncmp (">From ", line, 6) == 0)
-      {
-	loc = ftell (f);
 	continue; /* just ignore */
-      }
-      else if ((t = is_from (line, return_path, sizeof (return_path))))
+      else if (is_from (line, return_path, sizeof (return_path), &t))
       {
 	/* MH somtimes has the From_ line in the middle of the header! */
 	if (hdr && !hdr->received)
 	  hdr->received = t - mutt_local_tz (t);
-	loc = ftell (f);
 	continue;
       }
 
@@ -903,283 +1341,57 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs)
       break; /* end of header */
     }
 
+    *buf = '\0';
+
+    if (mutt_match_spam_list(line, SpamList, buf, sizeof(buf)))
+    {
+      if (!mutt_match_rx_list(line, NoSpamList))
+      {
+
+	/* if spam tag already exists, figure out how to amend it */
+	if (e->spam && *buf)
+	{
+	  /* If SpamSep defined, append with separator */
+	  if (SpamSep)
+	  {
+	    mutt_buffer_addstr(e->spam, SpamSep);
+	    mutt_buffer_addstr(e->spam, buf);
+	  }
+
+	  /* else overwrite */
+	  else
+	  {
+	    e->spam->dptr = e->spam->data;
+	    *e->spam->dptr = '\0';
+	    mutt_buffer_addstr(e->spam, buf);
+	  }
+	}
+
+	/* spam tag is new, and match expr is non-empty; copy */
+	else if (!e->spam && *buf)
+	{
+	  e->spam = mutt_buffer_from(NULL, buf);
+	}
+
+	/* match expr is empty; plug in null string if no existing tag */
+	else if (!e->spam)
+	{
+	  e->spam = mutt_buffer_from(NULL, "");
+	}
+
+	if (e->spam && e->spam->data)
+          dprint(5, (debugfile, "p822: spam = %s\n", e->spam->data));
+      }
+    }
+
     *p = 0;
     p++;
     SKIPWS (p);
     if (!*p)
       continue; /* skip empty header fields */
 
-    switch (tolower (line[0]))
-    {
-      case 'a':
-	if (mutt_strcasecmp (line+1, "pparently-to") == 0)
-	{
-	  e->to = rfc822_parse_adrlist (e->to, p);
-	  matched = 1;
-	}
-	else if (mutt_strcasecmp (line+1, "pparently-from") == 0)
-	{
-	  e->from = rfc822_parse_adrlist (e->from, p);
-	  matched = 1;
-	}
-	break;
-
-      case 'b':
-	if (mutt_strcasecmp (line+1, "cc") == 0)
-	{
-	  e->bcc = rfc822_parse_adrlist (e->bcc, p);
-	  matched = 1;
-	}
-	break;
-
-      case 'c':
-	if (mutt_strcasecmp (line+1, "c") == 0)
-	{
-	  e->cc = rfc822_parse_adrlist (e->cc, p);
-	  matched = 1;
-	}
-	else if (mutt_strncasecmp (line + 1, "ontent-", 7) == 0)
-	{
-	  if (mutt_strcasecmp (line+8, "type") == 0)
-	  {
-	    if (hdr)
-	      mutt_parse_content_type (p, hdr->content);
-	    matched = 1;
-	  }
-	  else if (mutt_strcasecmp (line+8, "transfer-encoding") == 0)
-	  {
-	    if (hdr)
-	      hdr->content->encoding = mutt_check_encoding (p);
-	    matched = 1;
-	  }
-	  else if (mutt_strcasecmp (line+8, "length") == 0)
-	  {
-	    if (hdr)
-	      hdr->content->length = atoi (p);
-	    matched = 1;
-	  }
-	  else if (mutt_strcasecmp (line+8, "description") == 0)
-	  {
-	    if (hdr)
-	    {
-	      safe_free ((void **) &hdr->content->description);
-	      hdr->content->description = safe_strdup (p);
-	      rfc2047_decode (hdr->content->description,
-			      hdr->content->description,
-			      mutt_strlen (hdr->content->description) + 1);
-	    }
-	    matched = 1;
-	  }
-	  else if (mutt_strcasecmp (line+8, "disposition") == 0)
-	  {
-	    if (hdr)
-	      parse_content_disposition (p, hdr->content);
-	    matched = 1;
-	  }
-	}
-	break;
-
-      case 'd':
-	if (!mutt_strcasecmp ("ate", line + 1))
-	{
-	  safe_free((void **)&e->date);
-	  e->date = safe_strdup(p);
-	  if (hdr)
-	    hdr->date_sent = mutt_parse_date (p, hdr);
-	  matched = 1;
-	}
-	break;
-
-      case 'e':
-	if (!mutt_strcasecmp ("xpires", line + 1) &&
-	    hdr && mutt_parse_date (p, NULL) < time (NULL))
-	  hdr->expired = 1;
-	break;
-
-      case 'f':
-	if (!mutt_strcasecmp ("rom", line + 1))
-	{
-	  e->from = rfc822_parse_adrlist (e->from, p);
-	  matched = 1;
-	}
-	break;
-
-      case 'i':
-	if (!mutt_strcasecmp (line+1, "n-reply-to"))
-	{
-	  if (hdr)
-	  {
-	    strfcpy (in_reply_to, p, sizeof (in_reply_to));
-	    rfc2047_decode (in_reply_to, in_reply_to,
-			    sizeof (in_reply_to));
-	  }
-	}
-	break;
-
-      case 'l':
-	if (!mutt_strcasecmp (line + 1, "ines"))
-	{
-	  if (hdr)
-	    hdr->lines = atoi (p);
-	  matched = 1;
-	}
-	break;
-
-      case 'm':
-	if (!mutt_strcasecmp (line + 1, "ime-version"))
-	{
-	  if (hdr)
-	    hdr->mime = 1;
-	  matched = 1;
-	}
-	else if (!mutt_strcasecmp (line + 1, "essage-id"))
-	{
-	  /* We add a new "Message-Id:" when building a message */
-	  safe_free ((void **) &e->message_id);
-	  e->message_id = extract_message_id (p);
-	  matched = 1;
-	}
-	else if (!mutt_strncasecmp (line + 1, "ail-", 4))
-	{
-	  if (!mutt_strcasecmp (line + 5, "reply-to"))
-	  {
-	    /* override the Reply-To: field */
-	    rfc822_free_address (&e->reply_to);
-	    e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
-	    matched = 1;
-	  }
-	  else if (!mutt_strcasecmp (line + 5, "followup-to"))
-	  {
-	    e->mail_followup_to = rfc822_parse_adrlist (e->mail_followup_to, p);
-	    matched = 1;
-	  }
-	}
-	break;
-
-      case 'r':
-	if (!mutt_strcasecmp (line + 1, "eferences"))
-	{
-	  mutt_free_list (&e->references);
-	  e->references = mutt_parse_references (p);
-	  matched = 1;
-	}
-	else if (!mutt_strcasecmp (line + 1, "eply-to"))
-	{
-	  e->reply_to = rfc822_parse_adrlist (e->reply_to, p);
-	  matched = 1;
-	}
-	else if (!mutt_strcasecmp (line + 1, "eturn-path"))
-	{
-	  e->return_path = rfc822_parse_adrlist (e->return_path, p);
-	  matched = 1;
-	}
-	else if (!mutt_strcasecmp (line + 1, "eceived"))
-	{
-	  if (hdr && !hdr->received)
-	  {
-	    char *d = strchr (p, ';');
-
-	    if (d)
-	      hdr->received = mutt_parse_date (d + 1, NULL);
-	  }
-	}
-	break;
-
-      case 's':
-	if (!mutt_strcasecmp (line + 1, "ubject"))
-	{
-	  if (!e->subject)
-	    e->subject = safe_strdup (p);
-	  matched = 1;
-	}
-	else if (!mutt_strcasecmp (line + 1, "ender"))
-	{
-	  e->sender = rfc822_parse_adrlist (e->sender, p);
-	  matched = 1;
-	}
-	else if (!mutt_strcasecmp (line + 1, "tatus"))
-	{
-	  if (hdr)
-	  {
-	    while (*p)
-	    {
-	      switch(*p)
-	      {
-		case 'r':
-		  hdr->replied = 1;
-		  break;
-		case 'O':
-		  if (option (OPTMARKOLD))
-		    hdr->old = 1;
-		  break;
-		case 'R':
-		  hdr->read = 1;
-		  break;
-	      }
-	      p++;
-	    }
-	  }
-	  matched = 1;
-	}
-	else if ((!mutt_strcasecmp ("upersedes", line + 1) ||
-	          !mutt_strcasecmp ("upercedes", line + 1)) && hdr)
-	  e->supersedes = safe_strdup (p);
-	break;
-
-      case 't':
-	if (mutt_strcasecmp (line+1, "o") == 0)
-	{
-	  e->to = rfc822_parse_adrlist (e->to, p);
-	  matched = 1;
-	}
-	break;
-
-      case 'x':
-	if (mutt_strcasecmp (line+1, "-status") == 0)
-	{
-	  if (hdr)
-	  {
-	    while (*p)
-	    {
-	      switch (*p)
-	      {
-		case 'A':
-		  hdr->replied = 1;
-		  break;
-		case 'D':
-		  hdr->deleted = 1;
-		  break;
-		case 'F':
-		  hdr->flagged = 1;
-		  break;
-		default:
-		  break;
-	      }
-	      p++;
-	    }
-	  }
-	  matched = 1;
-	}
-	    
-      default:
-	break;
-    }
-
-     /* Keep track of the user-defined headers */
-    if (!matched && user_hdrs)
-    {
-      if (last)
-      {
-	last->next = mutt_new_list ();
-	last = last->next;
-      }
-      else
-	last = e->userhdrs = mutt_new_list ();
-      line[strlen (line)] = ':';
-      last->data = safe_strdup (line);
-    }
-
-    loc = ftell (f);
+    matched = mutt_parse_rfc822_line (e, hdr, line, p, user_hdrs, weed, 1, &last);
+    
   }
 
   FREE (&line);
@@ -1189,28 +1401,11 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs)
     hdr->content->hdr_offset = hdr->offset;
     hdr->content->offset = ftell (f);
 
-    /* if an in-reply-to was given, check to see if it is in the references
-     * list already.  if not, add it so we can do a better job of threading.
-     */
-    if (in_reply_to[0] && (p = extract_message_id (in_reply_to)) != NULL)
-    {
-      if (!e->references ||
-	  (e->references && mutt_strcmp (e->references->data, p) != 0))
-      {
-	LIST *tmp = mutt_new_list ();
-
-	tmp->data = p;
-	tmp->next = e->references;
-	e->references = tmp;
-      }
-      else
-	safe_free ((void **) &p);
-    }
-
     /* do RFC2047 decoding */
     rfc2047_decode_adrlist (e->from);
     rfc2047_decode_adrlist (e->to);
     rfc2047_decode_adrlist (e->cc);
+    rfc2047_decode_adrlist (e->bcc);
     rfc2047_decode_adrlist (e->reply_to);
     rfc2047_decode_adrlist (e->mail_followup_to);
     rfc2047_decode_adrlist (e->return_path);
@@ -1220,7 +1415,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs)
     {
       regmatch_t pmatch[1];
 
-      rfc2047_decode (e->subject, e->subject, mutt_strlen (e->subject) + 1);
+      rfc2047_decode (&e->subject);
 
       if (regexec (ReplyRegexp.rx, e->subject, 1, pmatch, 0) == 0)
 	e->real_subj = e->subject + pmatch[0].rm_eo;
