@@ -1058,6 +1058,7 @@ ci_send_message (int flags,		/* send mode */
   /* save current value of "pgp_sign_as" */
   char *signas = NULL;
   char *tag = NULL, *err = NULL;
+  char *ctype;
 
   int rv = -1;
   
@@ -1122,7 +1123,9 @@ ci_send_message (int flags,		/* send mode */
     pbody->next = msg->content; /* don't kill command-line attachments */
     msg->content = pbody;
     
-    mutt_parse_content_type (ContentType, msg->content);
+    ctype = safe_strdup (ContentType);
+    mutt_parse_content_type (ctype, msg->content);
+    FREE (&ctype);
     msg->content->unlink = 1;
     msg->content->use_disp = 0;
     msg->content->disposition = DISPINLINE;
@@ -1350,7 +1353,7 @@ ci_send_message (int flags,		/* send mode */
   {
     struct stat st;
     time_t mtime = mutt_decrease_mtime (msg->content->filename, NULL);
-    
+
     mutt_update_encoding (msg->content);
 
     /*
@@ -1370,7 +1373,10 @@ ci_send_message (int flags,		/* send mode */
     {
       /* If the this isn't a text message, look for a mailcap edit command */
       if (mutt_needs_mailcap (msg->content))
-	mutt_edit_attachment (msg->content);
+      {
+	if (!mutt_edit_attachment (msg->content))
+          goto cleanup;
+      }
       else if (!Editor || mutt_strcmp ("builtin", Editor) == 0)
 	mutt_builtin_editor (msg->content->filename, msg, cur);
       else if (option (OPTEDITHDRS))
